# Copyright (c) 2015-present Stduino <service001@stduino.com>
#
# This file may be used under the terms of the GNU General Public License
# version 3.0 as published by the Free Software Foundation and appearing in
# the file LICENSE included in the packaging of this file.  Please review the
# following information to ensure the GNU General Public License version 3.0
# requirements will be met: http://www.gnu.org/copyleft/gpl.html.
#
# If you do not wish to use this file under the terms of the GPL version 3.0
# then you may purchase a commercial license.  For more information contact
# service001@stduino.com.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from __future__ import absolute_import
from .main import *

__version__ = '1.0.0'
__license__ = ''



# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""

# from six.moves import urllib  # 下载文件

#import zipfile  # 解压文件
# import shutil  # 用于压缩
# import subprocess
import os,stat

# import re
# import socket
#
# import zipfile


from PyQt5.QtWidgets import QMessageBox,QWidget,QPushButton,QVBoxLayout,QGridLayout,QTextEdit,QHBoxLayout,QGroupBox,QAbstractItemView,QListView

from PyQt5.QtCore import QStringListModel

# from function.conf import res
from .stdmsg import reso

from .stdedit import stdinit
import threading
from ..piobuilder.piopltinstall import PioPlatformInstall
from shutil import copytree,rmtree
# fo = open("./appearance/stduino_json/library.json", mode='r', encoding='UTF-8')
# st = fo.read()
# fo.close()
# libraries = json.loads(st)



class StdOpenExamples(QWidget):  # 安装库文件


    def __init__(self):
        try:
            super(StdOpenExamples, self).__init__()

            self.plat = PioPlatformInstall()
            self.add_lib_s_load_f = 0
            self.find_more_p = 1
            self.pio_libs = []
            self.pio_libs.append(0)
            self.lib_list = []
            self.example_list=[]
            self.platform_sec=None
            self.example_sec = None
            self.example_name=None
            self.examp_path=None

            self.initUi()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def initUi(self):
        try:
            self.bigEditor = QTextEdit()
            self.bigEditor.setReadOnly(True)

            #self.createGridGroupBox()
            #self.createGridGroupBox2()
            self.creatVboxGroupBox()
            self.creatVboxGroupBox2()
            mainLayout = QVBoxLayout()
            hboxLayout = QHBoxLayout()
            hboxLayout.addWidget(self.vboxGroupBox2)
            hboxLayout.addWidget(self.vboxGroupBox)

            #the_window.add_lib_s_load.connect(self.libr_load)

            self.install = QPushButton("请选择具体示例文件", self)
            self.install_return = QPushButton("退出当前界面", self)

            # self.find_more.setDisabled(True)
            self.install.setDisabled(True)
            self.install.clicked.connect(self.install_url)
            self.install_return.clicked.connect(self.install_ret)
            #self.install.setFixedWidth(120)
            #self.pro1 = QProgressBar(self)
            # self.pro1.setOrientation(Qt.Horizontal)
            # self.pro1.setMinimum(0)
            # self.pro1.setMaximum(0)
            #self.pro1.setVisible(False)

            glayout = QGridLayout()
            glayout.addWidget(self.install, 0, 0)

            glayout.addWidget(self.install_return, 0, 2)
            # 准备四个控件
            gwg = QWidget()
            # 使用四个控件设置局部布局
            gwg.setLayout(glayout)
            # 将四个控件添加到全局布局中


            #mainLayout.addWidget(self.gridGroupBox)
            mainLayout.addLayout(hboxLayout)
            #mainLayout.addWidget(self.gridGroupBox2)
            #mainLayout.addWidget(self.pro1)
            mainLayout.addWidget(gwg)
            #mainLayout.addWidget(self.install_return)
            self.setLayout(mainLayout)
        except:
            stdinit.std_signal_gobal.stdprintln()
    def libr_load(self):
        try:

            t1 = threading.Thread(target=self.library_load, name='li_load1')
            t1.setDaemon(True)
            t1.start()

        except:
            stdinit.std_signal_gobal.stdprintln()




    # def createGridGroupBox2(self):
    #     try:
    #         self.gridGroupBox2 = QGroupBox(reso.lib_version)
    #         self.layout = QGridLayout()
    #         # nameLabel = QLabel("发布版本时间")
    #         # self.vsion = QCheckBox(reso.select_version, self)
    #         # self.vsion.setChecked(True)
    #         # self.vsion.setEnabled(False)
    #         # self.vsion.stateChanged.connect(self.check_vsion)
    #         # self.combobox_1 = QComboBox(self)
    #         # self.combobox_1.setDisabled(True)
    #         self.layout.setSpacing(10)
    #         #self.layout.addWidget(self.vsion, 1, 0)
    #         #self.combobox_1.addItem("")
    #         #self.layout.addWidget(self.combobox_1, 1, 1)
    #         self.layout.setColumnStretch(1, 10)
    #         self.gridGroupBox2.setLayout(self.layout)
    #     except:
    #         stdinit.std_signal_gobal.stdprintln()


    def creatVboxGroupBox(self):
        try:
            self.vboxGroupBox = QGroupBox("对应平台示例项目")
            layout = QVBoxLayout()
            self.model_2 = QStringListModel(self)
            self.listview_2 = QListView(self)  # python3
            self.example_list = ["请先单击已安装平台"]
            self.model_2.setStringList(self.example_list)
            self.listview_2.setModel(self.model_2)

            self.listview_2.setEditTriggers(QAbstractItemView.NoEditTriggers)
            # self.listview_1.setFixedWidth(600)
            # self.listview_1.setFixedHeight(400)
            self.listview_2.clicked.connect(self.example_clicked)

            #self.bigEditor.setPlainText("请先单击选择已安装平台")
            layout.addWidget(self.listview_2)
            self.vboxGroupBox.setLayout(layout)

        except:
            stdinit.std_signal_gobal.stdprintln()


    def creatVboxGroupBox2(self):

        try:
            self.vboxGroupBox2 = QGroupBox("已安装平台")
            layout = QVBoxLayout()

            self.model_1 = QStringListModel(self)
            self.listview_1 = QListView(self)  # python3
            self.lib_list=["正在加载文件夹"]
            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)

            self.listview_1.setEditTriggers(QAbstractItemView.NoEditTriggers)
            #self.listview_1.setFixedWidth(600)
            #self.listview_1.setFixedHeight(400)
            self.listview_1.clicked.connect(self.clicked)
            layout.addWidget(self.listview_1)
            self.vboxGroupBox2.setLayout(layout)
                    # self.listview_1.doubleClicked.connect(lambda: self.change_func(self.listview_1))
                    # self.listview_1.doubleClicked.connect(self.cs)

                    #self.listview_1.clicked.connect(self.clicked)



        except:
            self.lib_list = [reso.no_net]
            self.model_1 = QStringListModel(self)
            self.listview_1 = QListView(self)  # python3
            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)
            self.listview_1.setEditTriggers(QAbstractItemView.NoEditTriggers)
            #self.listview_1.setFixedWidth(600)
            #self.listview_1.setFixedHeight(400)

            # self.item_list = ['item %s' % i for i in range(51)]  # 1
            stdinit.std_signal_gobal.stdprintln()

    def library_load(self):
        try:

            # print(libraries)
            example_path=stdinit.stdenv+"/.platformio/platforms"
            if os.path.exists(example_path):
                self.lib_list = os.listdir(example_path)
                self.install_return.setText("Finding Platforms File...")
                if len(self.lib_list) > 0:
                    self.model_1.setStringList(self.lib_list)
                    self.listview_1.setModel(self.model_1)
                    self.install_return.setText(reso.library_installer)
                else:
                    self.lib_list = ["暂未安装任何平台"]
                    self.model_1.setStringList(self.lib_list)
                    self.listview_1.setModel(self.model_1)
                    self.install_return.setText(reso.library_installer)
            else:
                self.lib_list = ["暂未安装任何平台"]
                self.model_1.setStringList(self.lib_list)
                self.listview_1.setModel(self.model_1)
                self.install_return.setText(reso.library_installer)

        except:
            stdinit.std_signal_gobal.stdprintln()

    def install_ret(self):
        try:

            stdinit.std_signal_gobal.std_main_ui_change(4)
            # the_window.m_stackedLayout.setCurrentIndex(0)
            # the_window.liAct2.setEnabled(True)

        except:
            stdinit.std_signal_gobal.stdprintln()
    def example_clicked(self,qModelIndex):
        try:

            n_lin=qModelIndex.row()
            self.example_sec = self.example_list[n_lin]#self.pio_plts[n_lin]['description']

            if self.platform_sec==None:
                return 0
            self.examp_path=stdinit.stdenv+"/.platformio/platforms/"+self.platform_sec+"/examples/"+self.example_sec
            if os.path.exists(self.examp_path):
                self.install.setDisabled(False)
                self.install.setText("打开该示例")
            else:
                self.install.setDisabled(True)
                self.install.setText("该示例不存在")

                #self.install_return.setText(reso.library_installer)
            # for i in
            # self.combobox_1.clear()
            #self.pio_libs[self.find_more_p]['items'][i]['name']
            # stdinit.plat_choose_name=self.pio_plts[n_lin]['name']
            # self.combobox_1.addItem(self.pio_plts[n_lin]['versions'][0])

            # if self.find_more.isEnabled() == False:
            #     self.find_more.setDisabled(False)


        except:
            stdinit.std_signal_gobal.stdprintln()
    def clicked(self, qModelIndex):

        try:
            self.install.setDisabled(True)
            self.install.setText("请选择具体示例")
            n_lin=qModelIndex.row()
            self.platform_sec = self.lib_list[n_lin]#self.pio_plts[n_lin]['description']
            examp_path=stdinit.stdenv+"/.platformio/platforms/"+self.platform_sec+"/examples"
            if os.path.exists(examp_path):
                self.example_list.clear()
                self.example_list=os.listdir(examp_path)
                self.model_2.setStringList(self.example_list)
                self.listview_2.setModel(self.model_2)
            else:
                self.example_list.clear()
                self.example_list.append("暂无示例")
                self.model_2.setStringList(self.example_list)
                self.listview_2.setModel(self.model_2)
                #self.install_return.setText(reso.library_installer)
            # for i in
            # self.combobox_1.clear()
            #self.pio_libs[self.find_more_p]['items'][i]['name']
            # stdinit.plat_choose_name=self.pio_plts[n_lin]['name']
            # self.combobox_1.addItem(self.pio_plts[n_lin]['versions'][0])

            # if self.find_more.isEnabled() == False:
            #     self.find_more.setDisabled(False)


        except:
            stdinit.std_signal_gobal.stdprintln()

    def remove_readonly(self, func, path, _):
        try:
            # "Clear the readonly bit and reattempt the removal"
            os.chmod(path, stat.S_IWRITE)
            func(path)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def delete_file(self, path):
        try:
            if os.path.isdir(path):
                try:
                    rmtree(path, onerror=self.remove_readonly)
                except:
                    stdinit.std_signal_gobal.stdprintln()
                    # self.listwidget.append("Delete error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder")
                    # self.add_lib_si(python3, 0, "Unexpected error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder")
                    # QMessageBox.warning(self, "Delete error",
                    #                     "Unexpected error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder",
                    #                     QMessageBox.Yes)
                    pass
            else:
                try:
                    os.remove(path)
                except:
                    try:
                        # shutil.rmtree(path, onerror=self.remove_readonly)
                        os.chmod(path, stat.S_IWRITE)
                        os.remove(path)
                    except:
                        stdinit.std_signal_gobal.stdprintln()
                pass
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()
    def thread_return(self,strin):
        self.install.setText(strin)
        self.install.setEnabled(True)
        self.listview_1.setEnabled(True)
    def open_example(self):
        try:
            if os.path.exists(self.example_name):
                self.delete_file(self.example_name)
                #os.makedirs(self.example_name)
                copytree(self.examp_path, self.example_name)
                stdinit.std_signal_gobal.std_main_ui_change(4)
                stdinit.std_signal_gobal.std_echo_msg(0,"示例文件已拷贝至项目文件夹下，请手动进行切换当前工作空间！")
            else:
                #os.makedirs(self.example_name)
                copytree(self.examp_path, self.example_name)
                stdinit.std_signal_gobal.std_main_ui_change(4)
                stdinit.std_signal_gobal.std_echo_msg(0, "示例文件已拷贝至项目文件夹下，请手动进行切换当前工作空间！")

        except:
            stdinit.std_signal_gobal.stdprintln()



    def install_url(self):

        # from multiprocessing import Process
        # p = Process(target=self.install_url1, args=(1,))
        # print(1)
        try:

            self.install.setEnabled(False)
            # self.listview_1.setEnabled(False)
            example_nam=self.platform_sec + "-" + self.example_sec

            self.example_name=stdinit.projects_dir+self.platform_sec+"-"+self.example_sec
            if os.path.exists(self.example_name):
                ok = QMessageBox.information(self,
                                             "提示",
                                             "该示例文件:"+example_nam+"已打开是否重新打开?" ,
                                             QMessageBox.Yes, QMessageBox.No)
                if ok == QMessageBox.No:
                    return 0

            t1 = threading.Thread(target=self.open_example, name='open_example')
            t1.setDaemon(True)
            t1.start()  # File_tree_init
            # self.my_thread = stdthread()  # 步骤2. 主线程连接子线,同时传递一个值给子线程
            # self.my_thread.std_signal.connect(self.thread_return)
            # stdinit.callch = 1
            # self.my_thread.start()  # 步骤3 子线程开始执行run函数

        except:
            stdinit.std_signal_gobal.stdprintln()
        #t1 = threading.Thread(target=pay_ticket, name='th1')



    def schedule(self,blocknum, blocksize, totalsize):
        try:
            # """
            # blocknum:当前已经下载的块
            # blocksize:每次传输的块大小
            # totalsize:网页文件总大小
            # """
            per = 100 * blocknum * blocksize / totalsize

            if per > 100:
                per = 100
                self.install_return.setText("Installing Library...")

            #self.add_lib_s.emit(2, int(per), "")  # 2进度条数值
        except:
            stdinit.std_signal_gobal.stdprintln()










import os
import subprocess
import time
import json
import sqlite3
from PyQt5.QtCore import QObject
from ..stdedit import stdinit
# from function.cores.stdauto.Singleton import Singleton
import threading

'''================================================================================'''
'''|                               CTAGS PARSER                                   |'''
'''================================================================================'''


# @Singleton
class Ctags_parser(QObject):

    def __init__(self, parent=None):
        super(Ctags_parser, self).__init__(parent)

        try:
            self.json_string = None
            self.json_data = None
            self.conn = None
            self.cursor = None
            stdinit.std_signal_gobal.jump_init.connect(self.go_jump_iit)
        except:
            stdinit.std_signal_gobal.stdprintln()
        #self.build_ctags()


    ''''''
    def init_tags(self):
        try:
            platform_tags = stdinit.stdenv + "/.stduino/session/tags/" + stdinit.platform + stdinit.framework + ".tag"
            p_tags = stdinit.stdenv + "/.stduino/session/tags"
            if os.path.exists(p_tags):
                pass
            else:
                os.makedirs(p_tags)
            if os.path.exists(platform_tags):
                self.deledatabase()
                self.bulid_ctags_database2(platform_tags)
            else:

                platform_json_path = stdinit.stdenv + "/.platformio/platforms/" + stdinit.platform + "/platform.json"
                # print(platform_json_path)
                if os.path.exists(platform_json_path):
                    fo = open(platform_json_path, mode='r', encoding='UTF-8')
                    st = fo.read()
                    fo.close()
                    psd = json.loads(st)

                    frame_name = psd["frameworks"]["arduino"]["package"]

                    frame_path = stdinit.stdenv + "/.platformio/packages/" + frame_name + "/cores"
                    if os.path.exists(frame_path):
                        self.deledatabase()
                        self.build_ctags1(platform_tags, frame_path)

        except:
            stdinit.std_signal_gobal.stdprintln()

    def go_jump_iit(self):
        try:
            t1 = threading.Thread(target=self.init_tags, name='init_tags' + stdinit.platform)
            t1.setDaemon(True)
            t1.start()  #
        except:
            stdinit.std_signal_gobal.stdprintln()
        #print(t1)



    def build_ctags1(self,platform_tags,path):#ctags -x --c-types=f   tree-1.6.0/tree.c
        try:
            #print("ctags_path")
            #print(ctags_path)
            # os.chdir("C:/Users/debug/.platformio/packages/framework-arduinoststm32/cores")
            # r0=time.time()
            #ctags_path = None
            # print("Launching ctags on: " + path)
            # r0 = time.time()#C:\Users\debug\.platformio\packages\framework-arduino-gd32v\cores
            if stdinit.platform_is == "Win":
                ctags_path = stdinit.abs_path + "/tool/packages/ctagswin32/"

            elif stdinit.platform_is == "Linux":
                pass
                return 0


            elif stdinit.platform_is == "Darwin":
                pass
                return 0

            else:
                print("Cannot indentify platform: " + stdinit.platform_is)
                ctags_path = None

            if os.path.exists(ctags_path) == False:

                return 0
            #print(ctags_path)

            #cmd = "ctags -R --output-format=json --fields=+n --c-types=+l " + '"' + path + '"'  # C:/Users/debug/.platformio/packages/framework-arduino-gd32v/cores"#C:/Users/debug/.platformio/packages/framework-arduinoststm32/cores"

            cmd = ctags_path + "ctags -R --output-format=json --fields=+n --c-types=+l " + '"' + path + '"'   # C:/Users/debug/.platformio/packages/framework-arduino-gd32v/cores"#C:/Users/debug/.platformio/packages/framework-arduinoststm32/cores"
            # cmd="C:/Users/debug/Documents/Stduino/ctags32/ctags --output-format=json ctags -I __THROW –file-scope=yes –langmap=c:+.h –languages=c,c++ –links=yes –c-kinds=+p --fields=+S -R C:/Users/debug/.platformio/packages/framework-arduinoststm32/cores*"
            proc = subprocess.Popen(cmd,
                                    stdout=subprocess.PIPE, shell=True)
            # proc = subprocess.Popen(["C:/Users/debug/Documents/Stduino/ctags32/ctags", "-R", "--output-format=json",
            #                          "--fields=+n", "--c-types=+l","C:/Users/debug/.platformio/packages/framework-arduinoststm32/cores"],
            #                         stdout=subprocess.PIPE, shell=True)

            time.sleep(0.1)
            #print("Waiting for ctags to finish")
            time.sleep(0.1)
            # r1 = time.time()
            # print(r1-r0)
            # print(proc.communicate()[0])
            # print(proc.communicate())
            # fo = open("./appearance/stduino_json/personal.json", mode='r', encoding='UTF-8')
            # st = fo.read()
            # fo.close()  # cmd_args["cmd_rtlib"]
            self.json_string = (proc.communicate()[0]).decode('utf-8')
            # print(self.json_string)
            # print("")
            # print("")

            self.build_ctags_database(platform_tags)
        except:
            stdinit.std_signal_gobal.stdprintln()



    ''''''
    def bulid_ctags_database2(self,platform_tags):
        try:

            try:

                fo = open(platform_tags, mode='r', encoding='UTF-8')
                st = fo.read()
                fo.close()
                self.json_data = json.loads(st)


            except:
                stdinit.std_signal_gobal.stdprintln()

            self.conn = sqlite3.connect(":memory:",check_same_thread=False)

            self.cursor = self.conn.cursor()
            self.cursor.execute("""
                       CREATE TABLE IF NOT EXISTS main
                       (_type TEXT, name TEXT, path TEXT, pattern TEXT, line TEXT,
                       typeref TEXT, kind TEXT, scope TEXT, scopeKind TEXT)
                   """)

            for entry in self.json_data:
                # print(entry)

                self.cursor.execute("""
                       INSERT INTO main
                       (_type, name, path, pattern, line, typeref, kind, scope, scopeKind)
                       VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                                    (entry["_type"] if "_type" in entry else None,
                                     entry["name"] if "name" in entry else None,
                                     entry["path"] if "path" in entry else None,
                                     entry["pattern"] if "pattern" in entry else None,
                                     entry["line"] if "line" in entry else None,
                                     entry["typeref"] if "typeref" in entry else None,
                                     entry["kind"] if "kind" in entry else None,
                                     entry["scope"] if "scope" in entry else None,
                                     entry["scopeKind"] if "scopeKind" in entry else None
                                     )
                                    )
        except:
            stdinit.std_signal_gobal.stdprintln()

    def build_ctags_database(self,platform_tags):
        try:
            self.json_string = self.json_string.replace("}\r\n", "},\r\n")
            self.json_string = "[" + self.json_string + "]"
            self.json_string = self.json_string.replace("},\r\n]", "}\r\n]")

            try:
                #print(self.json_string)
                fo = open(platform_tags, "w+", encoding='UTF-8')
                fo.write(self.json_string)
                fo.close()
                self.json_data = json.loads(self.json_string)

            except:
                stdinit.std_signal_gobal.stdprintln()

            self.conn = sqlite3.connect(":memory:",check_same_thread=False)

            self.cursor = self.conn.cursor()
            self.cursor.execute("""
                       CREATE TABLE IF NOT EXISTS main
                       (_type TEXT, name TEXT, path TEXT, pattern TEXT, line TEXT,
                       typeref TEXT, kind TEXT, scope TEXT, scopeKind TEXT)
                   """)

            for entry in self.json_data:
                # print(entry)

                self.cursor.execute("""
                       INSERT INTO main
                       (_type, name, path, pattern, line, typeref, kind, scope, scopeKind)
                       VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                                    (entry["_type"] if "_type" in entry else None,
                                     entry["name"] if "name" in entry else None,
                                     entry["path"] if "path" in entry else None,
                                     entry["pattern"] if "pattern" in entry else None,
                                     entry["line"] if "line" in entry else None,
                                     entry["typeref"] if "typeref" in entry else None,
                                     entry["kind"] if "kind" in entry else None,
                                     entry["scope"] if "scope" in entry else None,
                                     entry["scopeKind"] if "scopeKind" in entry else None
                                     )
                                    )
        except:
            stdinit.std_signal_gobal.stdprintln()




        ###

    ''''''

    def get_symbol_kind(self, name):
        try:
            data = None
            try:
                self.cursor.execute("SELECT name,kind FROM main WHERE name = \"{n}\"".format(n=name))
                data = self.cursor.fetchone()
            except:
                pass

            if data is None:
                return ("", "")
            else:
                return (data[0], data[1])  # 自动补全功能，返回的数据是否可重复利用
        except:
            pass

        ###

    ''''''


    def where_to_jump(self, name):
        try:




            self.cursor.execute("SELECT path,line FROM main WHERE name = \"{n}\"".format(n=name))
            data =self.cursor.fetchone()


        except:
            stdinit.std_signal_gobal.stdprintln()
        try:
            if data == None:
                return ("0", "0")
            else:
                return data
        except:
            stdinit.std_signal_gobal.stdprintln()


        ###

    ''''''
    def deledatabase(self):
        try:
            if self.conn is not None:
                self.conn.commit()
                self.conn.close()
        except:
            stdinit.std_signal_gobal.stdprintln()
    def __del__(self):
        try:
            if self.conn is not None:
                self.conn.commit()
                self.conn.close()
        except:
            stdinit.std_signal_gobal.stdprintln()
        #print("Database closed")

    ''''''


'''=== end Class ==='''


# -*- coding: utf-8 -*-
#import appearance.language.zh_hans

#import appearance.language.en

import os
import configparser
from platform import system
stdenv = None
if system() == "Windows" or system() == "cli":

    stdenv = os.environ["USERPROFILE"].replace('\\', '/')

elif system() == "Linux":

    stdenv = os.environ["HOME"].replace('\\', '/')

elif system() == "Darwin":

    stdenv = os.environ["HOME"].replace('\\', '/')
cf =configparser.ConfigParser()
config_p=stdenv+"/.stduino/session/mainconfig.cfg"
if os.path.exists(config_p):
    pass
else:
    config_p="./appearance/mainconfig.cfg"
cf.read(config_p)  # 读取配置文件内容



msg= cf.get('language', 'msg')  # 返回_a章节里面key为a_key2的值，返回为int类型
lastfile=cf.get('filepath', 'lastfile')  # 返回_a章节里面key为a_key2的值，返回为int类型
stdtype=cf.get('std_type', 'stdtype')
download_type=cf.get('std_type', 'download_type')

try:
    std_auto = int(cf.get('std_type', 'std_auto'))
except:

    cf.set("std_type", "std_auto", "1")  # 章节a里面添加一个key为b_key3，值为new-$r，如果key存在就更新key的值
    cf.write(open(config_p, "w"))  # 把修改写入到文件test.conf中
    std_auto = 1
gbk_type=cf.get('gbk_utf', 'type')


#此处读出的均为字符串
default_c=int(cf.get('default_c', 'c'))
#pathstaus=int(cf.get('path', 'staus'))
# pathstaus2=cf.get('path', 'staus2')
version_id=cf.get('default_c', 'version_id')
keywordversion_id=cf.get('default_c', 'keywordversion_id')
boardversion_id=cf.get('default_c', 'boardversion_id')
updateversion_id=cf.get('default_c', 'updateversion_id')
updatetime=cf.get('default_c', 'updatetime')
creatshortcut= cf.get('language', 'st1')  # 返回_a章节里面key为a_key2的值，返回为int类型

if default_c==0:
    default_main = cf.get('defaultd_style', 'main')
    default_text = cf.get('defaultd_style', 'text')
    default_comment = cf.get('defaultd_style', 'comment')
    default_font = cf.get('defaultd_style', 'font')
    default_mainkey1 = cf.get('defaultd_style', 'mainkey1')
    default_mainkey2 = cf.get('defaultd_style', 'mainkey2')
    default_kh = cf.get('defaultd_style', 'kh')
    default_zh_hans = cf.get('defaultd_style', 'zh_hans')
    default_num = cf.get('defaultd_style', 'num')
    default_include = cf.get('defaultd_style', 'include')
elif default_c==1:
    default_main = cf.get('defaultl_style', 'main')
    default_text = cf.get('defaultl_style', 'text')
    default_comment = cf.get('defaultl_style', 'comment')
    default_font = cf.get('defaultl_style', 'font')
    default_mainkey1 = cf.get('defaultl_style', 'mainkey1')
    default_mainkey2 = cf.get('defaultl_style', 'mainkey2')
    default_kh = cf.get('defaultl_style', 'kh')
    default_zh_hans = cf.get('defaultl_style', 'zh_hans')
    default_num = cf.get('defaultl_style', 'num')
    default_include = cf.get('defaultl_style', 'include')
else :
    default_main = cf.get('personal_style', 'main')
    default_text = cf.get('personal_style', 'text')
    default_comment = cf.get('personal_style', 'comment')
    default_font = cf.get('personal_style', 'font')
    default_mainkey1 = cf.get('personal_style', 'mainkey1')
    default_mainkey2 = cf.get('personal_style', 'mainkey2')
    default_kh = cf.get('personal_style', 'kh')
    default_zh_hans = cf.get('personal_style', 'zh_hans')
    default_num = cf.get('personal_style', 'num')
    default_include = cf.get('personal_style', 'include')

#test
#xyz=zh_hans.xyz

#img
#pathstaus2

# import getpass
# from function.conf import setup

# path = getpass.getuser()

# if path != pathstaus2:
#     creatshortcut = cf.get('language', 'st0')  # 返回_a章节里面key为a_key2的值，返回为int类型
#
#
#     pos = os.path.abspath('.').replace("\\","/")
#     library = pos + "/tool/Documents/libraries"
#     projects = pos + "/tool/Documents/projects"
#     setup.library(library)
#     setup.projects(projects)
#     setup.paths2(path)
#     pass
# else:
#     library = cf.get('path', 'library')
#     projects = cf.get('path', 'projects')
#     pass

img_fileclose = 'appearance/img/fileclose.png'
img_complex = 'appearance/img/img_complex3.png'
img_uploadfast = 'appearance/img/img_uploadfast.png'
img_upload = 'appearance/img/img_upload.png'
img_back = 'appearance/img/img_back.png'
img_forward = 'appearance/img/img_forward.png'
img_search = 'appearance/img/img_search.png'
img_save = 'appearance/img/img_save.png'
img_saveall = 'appearance/img/img_saveall.png'
img_open = 'appearance/img/img_open.png'
img_addfile = 'appearance/img/img_addfile.png'
img_serial = 'appearance/img/img_serial.png'
img_serialp = 'appearance/img/img_serialp.jpg'
img_clean = 'appearance/img/img_clean.png'
img_msg = 'appearance/img/img_msg.png'
img_help = 'appearance/img/img_help.png'
img_web = 'appearance/img/img_web.png'
img_about = 'appearance/img/img_about.png'
img_setup = 'appearance/img/img_setup.png'
img_fold = 'appearance/img/img_fold.png'
img_exit = 'appearance/img/img_exit.png'
img_lib = 'appearance/img/img_lib.png'
img_lib_file = 'appearance/img/img_lib_file.png'
img_feedback = 'appearance/img/feedback.png'
img_library='appearance/img/Library2.png'
img_library_add='appearance/img/library_add.png'
img_library_fold='appearance/img/library_fold.png'
img_example='appearance/img/example.png'
img_projects='appearance/img/img_projects.png'
img_projects2='appearance/img/img_projects2.png'
img_platform='appearance/img/platform.png'
img_package='appearance/img/package.png'
img_pio='appearance/img/piofavicon.ico'
img_heart='appearance/img/good.svg'
img_comment='appearance/img/comment.png'
img_uncomment='appearance/img/uncomment.png'
img_goto='appearance/img/goto.png'
img_backto='appearance/img/backto.png'
img = 'appearance/img/st.PNG'


#test
xyz=123

#menu

file='File'
import_lib='Import Library'
lib_example='Example'
editor='Editor'
project='Project'
tool='Tool'
help='Use Help'

#menufile

openrecent='OpenRecent'
projectfile='PorjectFile'
open_folder='Open Project Folder'
exit='Exit App'
commenter='Comment / Cancel Comment'
language='LangSwitcher'
#toobar

complex='Compile'
upload='Compile and Download'
uploadfast='Download'
clean='Clean'
new='New'
open='Open'
save='Save'
saveall='Save All'
serial='Serial'
serialp='Serial pic'
search='Search'
back='Back'
forward='Forward'
editstyle='Page setup'
install_driver='Manual Installation Driver'
choose_down_type='Select Download Method'
usb_link='Serial port'
st_link='St_link V2'
tcommunity='Technical community'
community_web='Community Website'
about='About'
li_development='Library development'
create_lib='New Library'
search_lib='Search Library'
open_lib='Open Library Folder'
choose_board='Select board：'
choose_port='Download Port：'
check_port='Check Port'
restart_effect='Restart Settings Effective'

lib_search='Input Library Name'
lib_list='Library List'
board_list='New Pro'
example_list='Example'
project_list='Projects'
debug_button='Debug'
platform_list='Platforms'
package_list='Plug-ins'
package_uninstall='Uninstall'
lib_detail='More information'
lib_select='Select File to Install'
lib_version='Version information'
select_version='Latest version'
install_lib='Confirm installation'
lib_find_more='Find More'
stduino_lib_install='Sduino package Install'
no_net='No network to access, please connect the network and try again!'
format1='Format'
paste='Paste'
day_model='Daytime Model'
night_model='Night Model'
person_model='Personal Model'
person_setting='Options'
edit_back_color='Edit color'
main_back_color='Text color'
commenter_back_color='Comment color'
confirm_change='Confirm'
concle_change='Cancel'
exit_this='Exit'
find_next='Find/Next'
replace_next='Replace/Next'
replace_all='Replace All'
match='Exact match'
feedback='Feedback'
serial_port='No Serial Port!'
on=' On '
download_type_1=' Download Type: '

complexing='Compiling：'
start_complex='Begin to compile...'
now_program='The current program：'
row_line='Row/Column:'
warning_now_line='Warning: row/column:'
complex_complete='Compile successfully!'
complex_failed='Compilation fails!'
clean_out='Clear success'
clean_failed='Clear failed'
serial_port_check='Please check that the download port is selected correctly!'
serial_port_is_normal='Check whether the serial port is normal！'
st_link_check='Please check if the connection between St_link and MCU is correct!'
download_start='Start the download...'
downloading='Downloading：'
downloaded='Download successful!'
download_failed='Download failed!'
tip='Tip'
lib_installed='Library installation successful，\nThe library name that the actual import refers to is under src.h！'
lib_install_failed='Library installation failed, please try again later！'
lib_install_info='The library is unavailable and cannot be downloaded！'
lib_download='Downloading：'
shouce='The local instruction manual is being developed！'
server='We will gradually improve to provide you with better service'
server_detail='Problem description\nOpinion suggestion\nYour email address'
feed_back='Problem of feedback！'
feed_back_thank='Thank you for your feedback! Your comments will become an important basis for us to improve our software and services. We welcome your continued attention and support us！'
feed_back_failed='Feedback submission failed, please try again later'
about_ide='About this software'
function_development='This function is under development！'
lib_share='Select the library to share'
language_choose='Stduino IDE Library development language'
language_select='Select the library development language'
cancel='cancel'
lib_name='Library file name'
file_name="File name:"
line_text='//One keyword per line, delete the line after completion'
save_tip='Whether to save the changes'
save_tip_file='Whether to save the file'
last_close='This is the last window that will close the software'
find_text_none='Search completed: no such keyword!'
change_text_none='Replacement completed: no such keyword!'
change_text_num='Replacement completed: total replacement'
mail_sended='Email sent successfully'
mail_send_failed='Mail delivery failed, please try again later'
serial_port_helper='Serial port assistant'
serial_open_failed='This serial port cannot be opened！'
serial_port_staus_o='Serial port status (opened)'
serial_port_staus_c='Serial port status (closed)'
input_data_16='Please enter hexadecimal data separated by Spaces!'
send_hex='Sending HEX['
send_dec='Sending DEC['
receive_hex='Receiving HEX['
receive_dec='Receiving DEC['
reset_up='Restart the software to take effect'
startup_failed='Software startup failure'
search_tip='Please go to the official website to search“Resolution of software failure to start”！'
web_site='Official website：'
file_save='File saving'
select_file='Select the file'

add_file="Add file to projects"
add_direc="Add Directory to projects"
re_name="Rename..."
delete_f_d="Delete file/directory from projects"

code_flash = "Flash:"
code_ram = " TotalRam:"
code_static = " Percentage:"
code_bits = "Bits"

project_set="New project configuration"
project_name="Project name"
board_core_type="Chip"
project_framework="Framework"
board_usb="USB support"
board_usb_speed="USB speed"
board_opt="Compile settings"
board_runtime="Runtime library"
board_upload="Download mode"
board_debug="Debug mode"
project_build="Create"
library_installer="Exit the current installation interface"
search_sucess="Search Sucessfully!"


# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
__author__ = 'Su Jin Qiang'


#import zipfile  # 解压文件


from .stdedit import stdinit
from PyQt5.QtCore import QObject
from .stdmsg import reso
from ..piobuilder.piorun import PioRunManage

class Stdmake(QObject):

    def __init__(self):
        super(Stdmake, self).__init__()
        self.Pio_run = PioRunManage()
    def std_run(self):
        try:

            stdinit.std_signal_gobal.std_process(1, reso.complexing)  # complexing
            self.Pio_run.pio_run()
            stdinit.std_signal_gobal.std_process(0, reso.complex_complete)
            #self.make_signal.emit(0)
            stdinit.std_signal_gobal.std_make_signal(0)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def std_upload(self):
        try:
            stdinit.std_signal_gobal.std_process(1, reso.downloading)  # complexing
            self.Pio_run.pio_upload()
            stdinit.std_signal_gobal.std_process(0, reso.downloaded)
            stdinit.std_signal_gobal.std_make_signal(0)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def std_upload_fast(self):
        try:
            stdinit.std_signal_gobal.std_process(1, reso.downloading)  # complexing
            self.Pio_run.pio_upload()
            stdinit.std_signal_gobal.std_process(0, reso.downloaded)
            stdinit.std_signal_gobal.std_make_signal(0)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def std_clean(self):
        try:
            stdinit.std_signal_gobal.std_process(1, reso.clean)  # complexing
            self.Pio_run.pio_clean()
            stdinit.std_signal_gobal.std_process(0, reso.clean)
            stdinit.std_signal_gobal.std_make_signal(0)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def std_device_list(self):
        try:
            self.Pio_run.pio_device_list()
            stdinit.std_signal_gobal.std_make_signal(1)
        except:
            stdinit.std_signal_gobal.stdprintln()




# !/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
__author__ = 'Su Jin Qiang'

# from PyQt5.QtWidgets import *
from PyQt5.QtWidgets import QWidget,QFormLayout,QPushButton,QLineEdit,QLabel
from PyQt5.QtGui import QIcon
from function.cores.stdmsg import reso
from PyQt5 import QtCore, QtWidgets
from function.cores.stdedit import stdinit

#import tempfile
# 开发本软件消耗了太多时间精力，一路走来，太不容易，写下此行留为纪念，尤其感谢几位学弟（林鉴波、梁莅、房杰、刘席鸣等一直以来的各方面支持）。——2019.10.22晚

#pos = os.path.abspath('.')
#path = pos + "\\tool\msy"

#os.chdir("C:\stwork\stdemo2019827\dist\Stduinodebug\main")  # 通过更改当前运行目录F:\BaiduNetdiskDownload\stpython\stdemo\main


class StdFind(QWidget):

    def __init__(self, parent=None):
        try:
            super(StdFind, self).__init__(parent)
            self.setGeometry(680, 300, 350, 100)
            self.setFixedSize(350, 130)
            self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
            self.setWindowIcon(QIcon("appearance/img/st.PNG"))

            # 表单布局
            layout = QFormLayout()

            # 创建按钮，当行文本框并建立按钮点击与槽函数的联系，添加到布局中
            self.btn1 = QPushButton(reso.find_next)
            self.le1 = QLineEdit()
            self.le2 = QLineEdit()
            self.qfind = QtWidgets.QCheckBox()
            self.qfind.setChecked(True)
            self.btn1.clicked.connect(lambda: self.getItem(0))

            layout.addRow(self.le1, self.btn1)

            # 创建按钮，当行文本框并建立按钮点击与槽函数的联系，添加到布局中
            self.btn2 = QPushButton(reso.replace_next)
            self.btn2.clicked.connect(lambda: self.getItem(1))

            layout.addRow(self.le2, self.btn2)

            # 创建按钮，当行文本框并建立按钮点击与槽函数的联系，添加到布局中
            self.btn3 = QPushButton(reso.replace_all)
            self.btn3.clicked.connect(lambda: self.getItem(2))

            lbl = QLabel(reso.match, self)
            lbl.setGeometry(QtCore.QRect(30, 81, 90, 20))
            layout.addRow(self.qfind, self.btn3)

            # 设置主窗口的布局及标题
            self.setLayout(layout)
            self.setWindowTitle(reso.search)

        except:
            stdinit.std_signal_gobal.stdprintln()


    def getItem(self, id):
        try:
            idd = id
            findtext = self.le1.text()
            changetext = self.le2.text()
            afind = self.qfind.isChecked()
            stdinit.std_signal_gobal.std_find(idd, changetext, findtext, afind)

            # the_window.Tofind(idd, changetext, findtext, afind)

            # the_window.__editor.
            # to_find_text = 'delay'
            # ASt=the_window.__editor.findFirst(to_find_text,True,True,True,True)
            # self.__editor.replaceSelectedText('a')
            # self.__editor.#setAlignment()
            # while (self.__editor.find(to_find_text,QTextDocument.FindBackward)):#QTextDocument.FindBackward()
            # self.__editor.findFirst(to_find_text)
        except:
            stdinit.std_signal_gobal.stdprintln()


        # QMessageBox.critical(self, "Port Error", "此串口不能被打开！")





# !/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
__author__ = 'Su Jin Qiang'


import os
import time
from PyQt5.QtGui import QTextCursor,QIcon
from .stdmsg import reso
import serial  # pyserial
from .stdedit import stdinit
import serial.tools.list_ports
from PyQt5.QtCore import QTimer
from .uiserial import Std_Serial_Ui
from PyQt5 import QtCore, QtWidgets
# from function import process
# dsd = process.ProgressBar_start() #启动第一时间无法保证显示


#import tempfile
# 开发本软件消耗了太多时间精力，一路走来，太不容易，写下此行留为纪念，尤其感谢几位学弟（林鉴波、梁莅、房杰、刘席鸣等一直以来的各方面支持）。——2019.10.22晚


#pos = os.path.abspath('.')
#path = pos + "\\tool\msy"

#os.chdir("C:\stwork\stdemo2019827\dist\Stduinodebug\main")  # 通过更改当前运行目录F:\BaiduNetdiskDownload\stpython\stdemo\main


class StdSerial(QtWidgets.QWidget,Std_Serial_Ui):

    def __init__(self,id):
        super(StdSerial, self).__init__()
        try:
            stdinit.std_signal_gobal.upload_Close_check.connect(self.port_close)
            self.setupUi(self)  ############
            self.num_id=id
            self.init()
            self.setWindowTitle('Stduino ' + reso.serial_port_helper)
            self.setWindowIcon(QIcon("appearance/img/st.PNG"))
            self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
            self.setStyleSheet("color:#2F4F4F; background-color:483D8B;")  ##########
            self.close_button.setStyleSheet("color:#000000; background-color:#458B00;")

            self.setFixedSize(730, 550)  # 480
            # self.setMinimumSize(730, 480)
            # self.setMaximumSize(830, 680)

            self.ser = serial.Serial()
            self.port_check()

            # 接收数据和发送数据数目置零
            self.data_num_received = 0

            self.lineEdit.setText(str(self.data_num_received))
            self.data_num_sended = 0
            self.lineEdit_2.setText(str(self.data_num_sended))
        except:
            stdinit.std_signal_gobal.stdprintln()


    def init(self):
        try:
            # 串口检测按钮
            self.s1__box_1.clicked.connect(self.port_check)

            # 串口信息显示
            self.s1__box_2.currentTextChanged.connect(self.port_imf)

            # 打开串口按钮
            self.open_button.clicked.connect(self.port_open)

            # 关闭串口按钮
            self.close_button.clicked.connect(self.port_close)

            # 发送数据按钮
            self.s3__send_button.clicked.connect(self.data_send)

            # 定时发送数据


            self.timer_send = QTimer()
            self.timer_send.timeout.connect(self.data_send)
            self.timer_send_cb.stateChanged.connect(self.data_send_timer)

            # 定时器接收数据
            self.timer = QTimer()
            self.timer.timeout.connect(self.data_receive)

            # 清除发送窗口
            # self.s3__clear_button.clicked.connect(self.send_data_clear)

            # 清除接收窗口
            self.s2__clear_button.clicked.connect(self.receive_data_clear)

            # dtr rts
            self.DTR.clicked.connect(self.dtr)
            self.RTS.clicked.connect(self.rts)
            self.time_dot.clicked.connect(self.time_dt)

            self.hex_send.clicked.connect(self.hex_d)
            self.s4__calc.clicked.connect(self.calc)
        except:
            stdinit.std_signal_gobal.stdprintln()
    def time_dt(self):
        pass

    def timer_del(self):
        try:
            if self.timer is not None:
                self.timer.stop()
            if self.timer_send is not None:
                self.timer_send.stop()
        except:
            stdinit.std_signal_gobal.stdprintln()



    # 串口检测
    def port_check(self):
        try:
            self.Com_Dict = {}
            port_list = list(serial.tools.list_ports.comports())
            self.s1__box_2.clear()
            for port in port_list:
                self.Com_Dict["%s" % port[0]] = "%s" % port[1]
                self.s1__box_2.addItem(port[0])
            if len(self.Com_Dict) == 0:
                self.state_label.setText(" 无串口")
        except:
            stdinit.std_signal_gobal.stdprintln()
        # 检测所有存在的串口，将信息存储在字典中


    # 串口信息
    def port_imf(self):
        try:
            # 显示选定的串口的详细信息
            imf_s = self.s1__box_2.currentText()
            if imf_s != "":
                self.state_label.setText(self.Com_Dict[self.s1__box_2.currentText()])

        except:
            stdinit.std_signal_gobal.stdprintln()


    # 打开串口
    def port_open(self):
        try:
            self.ser.port = self.s1__box_2.currentText()
            if "COM" in self.ser.port:
                pass
            else:
                stdinit.std_signal_gobal.std_echo_msg(1,"None Serial port to Open")
                return 0
            self.ser.baudrate = int(self.s1__box_3.currentText())
            self.s1__box_3.setEnabled(False)
            self.ser.bytesize = 8  # int(self.s1__box_4.currentText())
            self.ser.stopbits = 1  # int(self.s1__box_6.currentText())
            self.ser.parity = "N"  # self.s1__box_5.currentText()
            self.ser.open()
            # 串口复位
            time.sleep(0.01)
            self.ser.rts = True
            self.ser.dtr = True
            self.ser.dtr = False
            time.sleep(0.01)
            self.ser.rts = False
            time.sleep(0.01)
            self.ser.dtr = True
            time.sleep(0.01)
            self.ser.rts = True
            self.ser.rts = False
            self.timer.start(2)

            if self.ser.isOpen():
                self.open_button.setEnabled(False)
                # self.open_button.setStyle("background-color:483D8B;")
                self.close_button.setStyleSheet("color:#2F4F4F; background-color:#f0f0f0;")  ##########
                self.open_button.setStyleSheet("color:#000000; background-color:#458B00;")  ##########
                self.close_button.setEnabled(True)
                self.formGroupBox1.setTitle(reso.serial_port_staus_o)
        except:
            stdinit.std_signal_gobal.stdprintln()
            stdinit.std_signal_gobal.std_echo_msg(1, "None Serial port to Open")
            return None
        #print(123322)


        # self.ser.xonxoff=False

        # xonxoff = 0,  # enable software flow control
        # rtscts = 0,  # enable RTS/CTS flow control




        # 打开串口接收定时器，周期为2ms


    # 关闭串口
    def port_close(self):
        try:
            self.timer.stop()
            self.timer_send.stop()

            if self.ser.isOpen():
                try:
                    self.s1__box_3.setEnabled(True)
                    self.ser.close()
                except:
                    stdinit.std_signal_gobal.stdprintln()

            self.open_button.setEnabled(True)
            self.close_button.setStyleSheet(
                "color:#000000; background-color:#458B00;")  ##########setStyleSheet("color:#2F4F4F; background-color:483D8B;")  ##########
            self.open_button.setStyleSheet("color:#2F4F4F; background-color:#f0f0f0;")  ##########
            self.close_button.setEnabled(False)
            self.lineEdit_3.setEnabled(True)
            # 接收数据和发送数据数目置零
            # self.data_num_received = 0
            # self.lineEdit.setText(str(self.data_num_received))
            # self.data_num_sended = 0
            # self.lineEdit_2.setText(str(self.data_num_sended))
            self.formGroupBox1.setTitle(reso.serial_port_staus_c)

        except:
            stdinit.std_signal_gobal.stdprintln()


    # 发送数据
    def data_send(self):
        try:

            if self.ser.isOpen():
                input_s = self.s3__send_text.toPlainText()
                if input_s != "":
                    # 非空字符串
                    if self.hex_send.isChecked():

                        # hex发送
                        input_s = input_s.strip()
                        send_list = []
                        while input_s != '':
                            try:
                                num = int(input_s[0:2], 16)
                            except:
                                stdinit.std_signal_gobal.stdprintln()
                                return None
                            input_s = input_s[2:].strip()
                            send_list.append(num)
                        input_s = bytes(send_list)

                        # hex显示
                        data = input_s
                        out_s = ''
                        for i in range(0, len(data)):
                            out_s = out_s + '{:02X}'.format(data[i]) + ' '
                        if self.time_dot.isChecked():
                            self.s2__receive_text.append(
                                time.strftime("%H:%M:%S", time.localtime()) + reso.send_hex + out_s + ']')
                            # self.s2__receive_text.insertPlainText(time.strftime("%H:%M:%S", time.localtime()) + reso.receive_hex + out_s + ']' + '\r\n')
                            pass
                        else:
                            self.s2__receive_text.append(out_s)
                            pass

                    else:
                        # ascii发送
                        input_s = (input_s).encode("utf-8")

                        # hex显示
                        data = input_s
                        # 串口接收到的字符串为b'123',要转化成unicode字符串才能输出到窗口中去
                        if self.time_dot.isChecked():
                            self.s2__receive_text.append(
                                time.strftime("%H:%M:%S", time.localtime()) + reso.send_dec + data.decode(
                                    "utf-8") + ']')
                            # self.s2__receive_text.insertPlainText(time.strftime("%H:%M:%S", time.localtime()) + reso.receive_hex + out_s + ']' + '\r\n')
                            pass
                        else:
                            self.s2__receive_text.append(data.decode("utf-8"))
                            pass

                    num = self.ser.write(input_s)
                    self.data_num_sended += num
                    self.lineEdit_2.setText(str(self.data_num_sended))

                    # 获取到text光标
                    textCursor = self.s2__receive_text.textCursor()
                    # 滚动到底部
                    textCursor.movePosition(textCursor.End)
                    # 设置光标到text中去
                    self.s2__receive_text.setTextCursor(textCursor)
            else:
                pass
        except:
            stdinit.std_signal_gobal.stdprintln()



    # 接收数据
    def data_receive(self):


        try:
            num = self.ser.inWaiting()
        except:
            stdinit.std_signal_gobal.stdprintln()
            self.port_close()
            return None
        try:
            if num > 0:
                time.sleep(0.2)
                #刷新速度过快，易产生接收分离的情况，这样就会出现解码错误

                data = self.ser.read(num)
                num = len(data)
                # hex显示
                if self.hex_receive.checkState():
                    out_s = ''
                    for i in range(0, len(data)):
                        out_s = out_s + '{:02X}'.format(data[i]) + ' '
                    if self.time_dot.isChecked():
                        self.s2__receive_text.append(
                            time.strftime("%H:%M:%S", time.localtime()) + reso.receive_hex + out_s + ']')
                        # self.s2__receive_text.insertPlainText(time.strftime("%H:%M:%S", time.localtime()) + reso.receive_hex + out_s + ']' + '\r\n')
                        pass
                    else:
                        self.s2__receive_text.append(out_s)
                        pass
                else:
                    if self.time_dot.isChecked():
                        self.s2__receive_text.append(
                            time.strftime("%H:%M:%S", time.localtime()) + reso.receive_dec + data.decode(
                                "utf-8") + ']')
                        pass
                    else:

                        self.s2__receive_text.moveCursor(QTextCursor.End)
                        self.s2__receive_text.insertPlainText(data.decode("utf-8", 'ignore'))

                        pass

                self.data_num_received += num
                self.lineEdit.setText(str(self.data_num_received))

                # 获取到text光标
                textCursor = self.s2__receive_text.textCursor()
                # 滚动到底部
                textCursor.movePosition(textCursor.End)
                # 设置光标到text中去
                self.s2__receive_text.setTextCursor(textCursor)
            else:
                pass
        except:
            stdinit.std_signal_gobal.stdprintln()
            #print("Unexpected error:", sys.exc_info()[1])  # 错误内容

    # 定时发送数据
    def data_send_timer(self):
        try:
            if self.timer_send_cb.isChecked():
                self.timer_send.start(int(self.lineEdit_3.text()))
                self.lineEdit_3.setEnabled(False)
            else:
                self.timer_send.stop()
                self.lineEdit_3.setEnabled(True)
        except:
            stdinit.std_signal_gobal.stdprintln()


        # dtr

    def dtr(self):
        try:
            if self.DTR.isChecked():
                if self.ser.isOpen():
                    self.ser.dtr = False

            else:
                if self.ser.isOpen():
                    self.ser.dtr = True

        except:
            stdinit.std_signal_gobal.stdprintln()


    def rts(self):
        try:
            if self.RTS.isChecked():
                if self.ser.isOpen():
                    self.ser.rts = False

            else:
                if self.ser.isOpen():
                    self.ser.rts = True
        except:
            stdinit.std_signal_gobal.stdprintln()


    def hex_d(self):
        try:
            if self.hex_send.isChecked():
                self.s3__send_text.setText("FF 00 30")

            else:
                self.s3__send_text.setText("123456")
        except:
            stdinit.std_signal_gobal.stdprintln()


    def calc(self):
        try:
            os.popen("calc.exe")
        except:
            stdinit.std_signal_gobal.stdprintln()



    # 清除显示
    # def send_data_clear(self):
    #     self.s3__send_text.setText("")

    def receive_data_clear(self):
        try:
            self.data_num_sended = 0
            self.lineEdit_2.setText(str(self.data_num_sended))
            self.data_num_received = 0
            self.lineEdit.setText(str(self.data_num_received))
            self.s2__receive_text.setText("")
        except:
            stdinit.std_signal_gobal.stdprintln()


    def closeEvent(self, event):
        try:
            if self.ser.isOpen():
                self.port_close()
            stdinit.serialnumber.remove(self.num_id)
            event.accept()
        except:
            stdinit.std_signal_gobal.stdprintln()





        #the_window.status2.setText(text + reso.on + the_window.wcombo.currentText() + reso.download_type_1 + 'USB_ISP')
        #the_window.status2.setText()




# !/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2015-present Stduino <service001@stduino.com>
#
# This file may be used under the terms of the GNU General Public License
# version 3.0 as published by the Free Software Foundation and appearing in
# the file LICENSE included in the packaging of this file.  Please review the
# following information to ensure the GNU General Public License version 3.0
# requirements will be met: http://www.gnu.org/copyleft/gpl.html.
#
# If you do not wish to use this file under the terms of the GPL version 3.0
# then you may purchase a commercial license.  For more information contact
# service001@stduino.com.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# 开发本软件消耗了太多时间精力，一路走来，太不容易，写下此行留为纪念，尤其感谢几位学弟（林鉴波、梁莅、房杰、刘席鸣等一直以来的各方面支持）。——2019.10.22晚


__author__ = 'Su Jin Qiang'

import os
import sys
import re
from function.cores.stdedit import stdinit

pos = sys.prefix.replace("\\", "/")
if re.search("main", pos) == None:
    pos = os.path.abspath('.')

    pass
os.chdir(pos)

pos = os.path.abspath('.').replace("\\","/")
from PyQt5.QtWidgets import QApplication,QFileDialog,QMessageBox,QMainWindow,QLabel,QSizePolicy,QVBoxLayout,QWidget
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont
from function.cores.stdmsg import reso
import qdarkstyle

from function.cores.stdserial import StdSerial
from function.cores.stdpersonal import StdBackColor
from function.cores.stdedit.stdmainui import StdMainWindow
from function.cores.stdedit.stdfind import StdFind
from function.cores.stdboards import StdFindBoards
from function.cores.stdedit.stdfile import StdPathHandler
from function.cores.stdmakeboards import StdMakeBoard
from function.cores.stdmake import Stdmake
from function.piobuilder.pioinstall import PioInstall
from function.piobuilder.piopltinstall import PioPlatformInstall
from function.cores.stdauto.ctags_parser import Ctags_parser

import threading
from shutil import copy2
# import time
# import webview
#
# from multiprocessing import Process, Queue
# que1 = Queue(10)
# def worker(q):
#     while True:
#         time.sleep(1)
#         print(1)
#
#         pass

        # print(stdinit.Std_test)
        # print('queue get: ', q.get())
        # print(121)

    # print(1212)
#     while True:
#         ress=que1.get()
#
#         if ress=="help":
#             webview.create_window('Stduino使用教程','http://www.stduino.com/forum.php?mod=viewthread&tid=105',text_select=True,on_top=True)
#             webview.start()
#
#
#         elif ress=="quan":
#             webview.create_window('额外省','http://taobao.stduino.com', text_select=True,on_top=True)
#             webview.start()
#         time.sleep(0.1)


version_c = "1.10.4内测版"#'1.01'



################################################
#######创建主窗口
################################################
class FirstMainWindow(QMainWindow):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setWindowTitle(reso.startup_failed)

        ###### 创建界面 ######
        self.centralwidget = QWidget()
        self.setCentralWidget(self.centralwidget)
        self.Layout = QVBoxLayout(self.centralwidget)

        # 设置顶部三个按钮
        # self.topwidget = QWidget()
        # self.Layout.addWidget(self.topwidget)
        # self.buttonLayout = QHBoxLayout(self.topwidget)
        #
        # self.pushButton1 = QPushButton()
        # self.pushButton1.setText("创建新项目文件")
        # self.buttonLayout.addWidget(self.pushButton1)
        #
        # self.pushButton2 = QPushButton()
        # self.pushButton2.setText("打开已有项目文件")
        # self.buttonLayout.addWidget(self.pushButton2)
        #
        # self.pushButton3 = QPushButton()
        # self.pushButton3.setText("配置软件")
        # self.buttonLayout.addWidget(self.pushButton3)

        # 设置中间文本
        self.label = QLabel()
        self.label.setText(reso.search_tip)
        self.label.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
        self.label.setAlignment(Qt.AlignCenter)
        self.label.setFont(QFont("Roman times", 10, QFont.Bold))
        self.Layout.addWidget(self.label)

        # 设置状态栏
        self.statusBar().showMessage(reso.web_site + 'www.stduino.com')
        self.setGeometry(500, 300, 600, 300)

    def on_pushButton1_clicked(self):

        # myGUI = CustomMainWindow()
        try:
            pos = os.path.abspath('.')
            path = pos + '.\\appearance\projects'
            fileName, ok2 = QFileDialog.getSaveFileName(self,
                                                        reso.file_save,
                                                        path,
                                                        "Target File(*.ino)")

            # print(fileName)
            fo = open(fileName, "wb")
            fo.close()
            st = ''
            the_window = StdMainWindow()
            the_window.fileName = fileName

            # the_window =SecondWindow()
            self.windowList.append(the_window)  ##注：没有这句，是不打开另一个主界面的！
            # the_window.EditorShow(st)
            the_window.setWindowTitle("Stduino IDE " + version_c + " [" + fileName + ']')  # 文件名设置
            self.close()
            the_window.show()
        # except OSError as err:
        #    print("OS error: {0}".format(err))
        # except ValueError:
        #   print("Could not convert data to an integer.")
        except:
            stdinit.std_signal_gobal.stdprintln()
            # print("Unexpected error:", sys.exc_info()[1]) #错误内容

    # 按钮二：打开现有项目文件
    def on_pushButton2_clicked(self):

        try:
            pos = os.path.abspath('.')
            path = pos + '.\\appearance\projects'
            fileName, filetype = QFileDialog.getOpenFileName(self,
                                                             reso.select_file,
                                                             path,
                                                             "Target File(*.ino)")  # 设置文件扩展名过滤,注意用双分号间隔
            fo = open(fileName, mode='r', encoding='UTF-8')
            st = fo.read()
            fo.close()

            # print(fileName)
            # self.label.setText(st
            the_window = StdMainWindow()
            the_window.fileName = fileName
            the_window.setWindowTitle("Stduino IDE " + version_c + " [" + fileName + ']')  # 文件名设置

            # the_window =SecondWindow()
            self.windowList.append(the_window)  ##注：没有这句，是不打开另一个主界面的！
            the_window.EditorShow(st)
            self.close()
            the_window.show()
            # except OSError as err:
            #    print("OS error: {0}".format(err))
            # except ValueError:
            #   print("Could not convert data to an integer.")
        except:
            stdinit.std_signal_gobal.stdprintln()
            #print("Unexpected error:", sys.exc_info()[1])  # 错误内容

        # the_dialog = TestdemoDialog()
        # if the_dialog.exec_() == QDialog.Accepted:
        # pass

    # 按钮三：打开提示框
    def on_pushButton3_clicked(self):
        QMessageBox.information(self, reso.tip, reso.function_development)
        # QMessageBox.question(self, "提示", "这是question框！")
        # QMessageBox.warning(self, "提示", "这是warning框！")
        # QMessageBox.about(self, "提示", "这是about框！")

def is_fast_file_in():

    try:
        if stdinit.platform_is=="Win":
            ideini = stdinit.stdenv + "/.stduino/packages/pioenv/Lib/site-packages/platformio"
            if os.path.exists(ideini):
                pass
            else:
                return 0
            ideini = stdinit.stdenv + "/.stduino/packages/pioenv/Lib/site-packages/stdini.pp"

            if os.path.exists(ideini):
                pass
            else:
                regi_p = stdinit.stdenv + "/.stduino/packages/pioenv/Lib/site-packages/platformio"
                regi = stdinit.stdenv + "/.stduino/packages/pioenv/Lib/site-packages/platformio/package/manager/_registry.py"
                if os.path.exists(regi):



                    os.remove(regi)
                    copy2(stdinit.abs_path + "/tool/packages/python3/Scripts/stdini.py",
                          stdinit.stdenv + "/.stduino/packages/pioenv/Lib/site-packages/platformio/stdini.pp")
                    copy2(stdinit.abs_path + "/tool/packages/python3/Scripts/stdload.py", regi)
                else:

                    copy2(stdinit.abs_path + "/tool/packages/python3/Scripts/stdini.py",
                          stdinit.stdenv + "/.stduino/packages/pioenv/Lib/site-packages/platformio/stdini.pp")
                    copy2(stdinit.abs_path + "/tool/packages/python3/Scripts/stdload.py", regi)
        elif stdinit.platform_is == "Linux":
            pass
            #ctags_path = stdinit.abs_path + "/tool/packages/ctagslin32/"
        elif stdinit.platform_is == "Darwin":
            pass
        else:
            print("Cannot indentify platform: " + stdinit.platform_is)

    except:
        #print("Cannot indentify platform: " + stdinit.platform_is)
        stdinit.std_signal_gobal.stdprintln()


            #创建快捷方式
    # pos = os.path.abspath('.')
    #
    # if res.creatshortcut == "1":
    #
    #     try:
    #         desk_path = stdinit.stdenv + "\Desktop"
    #         isExists = os.path.exists(desk_path)
    #         pos_path = pos[:-5]
    #         pos_name = pos_path + "\Stduino.exe"
    #
    #         if isExists:
    #             os.chdir(desk_path)  # 通过更改当前运行目录
    #             shell = client.Dispatch("WScript.Shell")
    #             shortcut = shell.CreateShortCut("Stduino.lnk")
    #             shortcut.WorkingDirectory = os.path.abspath(pos_path)  # 设置快捷方式的起始位置, 不然会出现找不到辅助文件的情况
    #             shortcut.TargetPath = os.path.abspath(pos_name)
    #             # shortcut.WorkingDirectory = "C:\stwork\stdemo2019827\dist\Stduino1.0"  # 设置快捷方式的起始位置, 不然会出现找不到辅助文件的情况
    #             # shortcut.TargetPath = "C:\stwork\stdemo2019827\dist\Stduino1.0\Stduino.exe"
    #             shortcut.save()
    #             os.chdir(pos)  # 通过更改当前运行目录
    #             # QMessageBox.warning(self, "快捷方式创建", "在桌面创建快捷方式\n Create shortcut on Desktop", QMessageBox.Yes)
    #         else:
    #             shell = client.Dispatch("WScript.Shell")
    #             shortcut = shell.CreateShortCut("Stduino.lnk")
    #             shortcut.WorkingDirectory = os.path.abspath(pos_path)  # 设置快捷方式的起始位置, 不然会出现找不到辅助文件的情况
    #             shortcut.TargetPath = os.path.abspath(pos_name)
    #             # shortcut.WorkingDirectory = "C:\stwork\stdemo2019827\dist\Stduino1.0"  # 设置快捷方式的起始位置, 不然会出现找不到辅助文件的情况
    #             # shortcut.TargetPath = "C:\stwork\stdemo2019827\dist\Stduino1.0\Stduino.exe"
    #             shortcut.save()
    #             QMessageBox.warning(self, "快捷方式创建",
    #                                 "已在" + pos_path + "创建快捷方式,请自行拷贝至桌面\n has Created shortcut on " + pos_path,
    #                                 QMessageBox.Yes)
    #
    #     except:
    #         QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdmainui0049！\n You can search it in stduino.com",
    #                             QMessageBox.Yes)




def view_init():
    pass

# from PyQt5.QtCore import QUrl
# import time
# from PyQt5.QtWidgets import *
# from PyQt5.QtWebEngineWidgets import QWebEngineView
#
# class Stduser():
#   def __init__(self):
#     super(Stduser, self).__init__()
#     self.browser=QWebEngineView()
#     #加载外部的web界面
#     self.browser.load(QUrl('http://api.stduino.com/users'))
#     time.sleep(10)
#
#     del self.browser
#     del self

if __name__ == '__main__':  # main函数
    app = QApplication(sys.argv)
    try:

        t1 = threading.Thread(target=is_fast_file_in, name='is_fast_file_in')
        t1.setDaemon(True)
        t1.start()#
        # print(2222)
        # que1.put("help")
        # p = Process(target=worker, args=(que1,))
        # p.daemon = True
        # p.start()
        # print(2323)
        # time.sleep(1)
        # que1.put("help")
        # time.sleep(1)
        # que1.put("help")
        #stdinit.Std_help.put("help")

        app.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())#qdarkgraystyle
        #app.setStyleSheet(qdarkgraystyle.load_stylesheet_pyqt5())

        # t1 = threading.Thread(target=view_init, name='is_fast_file_in')
        # t1.setDaemon(True)
        # t1.start()  # view_init

        stdinit.goto_init = Ctags_parser()
        stdinit.Pio_install = PioInstall()
        stdinit.Piopalt_install = PioPlatformInstall()
        stdinit.File_tree_view = StdPathHandler()
        stdinit.find_boards = StdFindBoards()
        stdinit.Std_Serial_Tool1 = StdSerial(0)
        stdinit.Std_Serial_Tool2 = StdSerial(1)
        stdinit.Set_Back_Color1 = StdBackColor()
        stdinit.Std_Find = StdFind()
        stdinit.Std_makeboards = StdMakeBoard()

        stdinit.Std_Make = Stdmake()
        std_main = StdMainWindow()
        std_main.show()
        std_main.readSession()
        stdinit.std_signal_gobal.all_stdenvir_init()


    except:
        stdinit.std_signal_gobal.stdprintln()
        std_main = FirstMainWindow()
        std_main.show()
        sys.exit(app.exec_())

    sys.exit(app.exec_())


    #setCompleter 自动补全实现方案 两个 一个textedit 加上setCompleter  二通过listview +查找工能

    #主进程1  负责ui及文件存储 窗口管理、进程间通信、自动更新等全局任务
    #主进展2 make相关及debug
    #rp Search文件搜索进程
    #plug子进程

''''''


# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
# !/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
__author__ = 'Su Jin Qiang'

import os
from .stdmsg import reso
from PyQt5.QtWidgets import QMessageBox,QCompleter, QComboBox,QWidget,QLabel, QLineEdit,QPushButton,QGridLayout
import threading
from .stdedit import stdinit
from ..piobuilder.pioproject import PioProjectManage
from PyQt5.QtCore import Qt, QSortFilterProxyModel

# 开发本软件消耗了太多时间精力，一路走来，太不容易，写下此行留为纪念，尤其感谢几位学弟（林鉴波、梁莅、房杰、刘席鸣等一直以来的各方面支持）。——2019.10.22晚

#pos = os.path.abspath('.')
#path = pos + "\\tool\msy"

#os.chdir("C:\stwork\stdemo2019827\dist\Stduinodebug\main")  # 通过更改当前运行目录F:\BaiduNetdiskDownload\stpython\stdemo\main


class StdComboBox(QComboBox):
    def __init__(self, parent=None):
        super(StdComboBox, self).__init__(parent)

        self.setFocusPolicy(Qt.StrongFocus)
        self.setEditable(True)


        # add a filter model to filter matching items
        self.pFilterModel = QSortFilterProxyModel(self)
        self.pFilterModel.setFilterCaseSensitivity(Qt.CaseInsensitive)
        self.pFilterModel.setSourceModel(self.model())

        # add a completer, which uses the filter model
        self.completer = QCompleter(self.pFilterModel, self)
        # always show all (filtered) completions
        self.completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
        self.setCompleter(self.completer)

        # connect signals
        self.lineEdit().textEdited.connect(self.pFilterModel.setFilterFixedString)
        self.completer.activated.connect(self.on_completer_activated)


    # on selection of an item from the completer, select the corresponding item from combobox
    def on_completer_activated(self, text):
        if text:
            index = self.findText(text)
            self.setCurrentIndex(index)
            self.activated[str].emit(self.itemText(index))


    # on model change, update the models of the filter and completer as well
    def setModel(self, model):
        super(StdComboBox, self).setModel(model)
        self.pFilterModel.setSourceModel(model)
        self.completer.setModel(self.pFilterModel)


    # on model column change, update the model column of the filter and completer as well
    def setModelColumn(self, column):
        self.completer.setCompletionColumn(column)
        self.pFilterModel.setFilterKeyColumn(column)
        super(StdComboBox, self).setModelColumn(column)


class StdFindBoards(QWidget):

    # Create a Json file for a better path management

    def __init__(self):
        try:
            self.pio_pro = PioProjectManage()
            # self.staus_change.connect(self.staus_change.emit)
            # if

            super(StdFindBoards, self).__init__()
            # self.setWindowOpacity(0.7)
            # self.setGeometry(50, 50, 50, 30)
            # self.my_signal_findboard.connect(self.sig_find_boards)
            # self.setFixedSize(420, 220)
            # self.setWindowTitle('New Project')
            # 置顶及去标题栏
            self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint)
            # self.setWindowIcon(QIcon("appearance/img/st.PNG"))
            self.setStyleSheet('background: DimGrey')

            # self.centralwidget = QtWidgets.QWidget(self)
            # self.centralwidget.setObjectName("centralwidget")
            self.project_label = QLabel(self)
            # self.begin_label.setGeometry(QtCore.QRect(20, 5, 225, 22))
            self.project_label.setText(reso.project_set)
            self.project_label.setObjectName("project_label")

            self.project_name_label = QLabel(self)

            self.project_name_label.setMinimumWidth(360)
            # self.board_label.setGeometry(QtCore.QRect(20, 40, 225, 22))
            self.project_name_label.setText(reso.project_name)
            self.project_name_label.setObjectName("project_name_label")
            self.project_nameEdit = QLineEdit("")

            self.board_label = QLabel(self)
            self.board_label.setMinimumWidth(360)
            # self.board_label.setGeometry(QtCore.QRect(20, 120, 225, 22))
            self.board_label.setText(reso.board_core_type)
            self.board_label.setObjectName("board_label")
            self.board = StdComboBox()
            self.board.setEditable(True)
            self.board.setMinimumWidth(360)
            self.board.setObjectName("board")
            self.board.activated.connect(self.to_board)

            self.framework_label = QLabel(self)
            self.framework_label.setMinimumWidth(360)
            # self.framework_label.setGeometry(QtCore.QRect(20, 200, 225, 22))
            self.framework_label.setText(reso.project_framework)
            self.framework_label.setObjectName("framework_label")
            self.framework = QComboBox(self)
            self.framework.setMinimumWidth(360)

            self.upload_method_label = QLabel(self)
            self.upload_method_label.setMinimumWidth(360)
            # self.upload_method_label.setGeometry(QtCore.QRect(20, 600, 225, 22))
            self.upload_method_label.setText(reso.board_debug)
            self.upload_method_label.setObjectName("upload_method_label")
            self.upload_method = QComboBox(self)
            self.upload_method.setMinimumWidth(360)
            # self.upload_method.activated.connect(lambda: self.to_upload_method(self.upload_method.currentText()))
            self.upload_method.setObjectName("upload_method")
            self.project_build = QPushButton(reso.project_build, self)
            self.project_build.clicked.connect(self.project_build_f)
            self.project_build.setStyleSheet('background: SlateGrey')
            self.project_build.setEnabled(False)
            self.project_cancel = QPushButton(reso.cancel, self)
            self.project_cancel.setStyleSheet('background: SlateGrey')
            self.project_cancel.clicked.connect(self.project_build_c)
            self.line_label = QLabel(self)
            self.line_label.setMinimumWidth(360)

            # self.pio_pro_boards = self.pio_pro.project_boards()

            # wlayout = QVBoxLayout()

            # 局部布局：水平，垂直，网格，表单
            glayout = QGridLayout()
            # line edit
            # LineEdit1 = QLineEdit()

            glayout.addWidget(self.project_label, 1, 0)  # name platform board fromwork  下载方式
            glayout.addWidget(self.project_name_label, 2, 0)
            glayout.addWidget(self.project_nameEdit, 3, 0)
            glayout.addWidget(self.board_label, 4, 0)

            glayout.addWidget(self.board, 5, 0)
            glayout.addWidget(self.framework_label, 6, 0)
            glayout.addWidget(self.framework, 7, 0)
            glayout.addWidget(self.upload_method_label, 8, 0)

            glayout.addWidget(self.upload_method, 9, 0)
            glayout.addWidget(self.line_label, 10, 0)
            glayout.addWidget(self.project_build, 11, 0)
            glayout.addWidget(self.project_cancel, 11, 1)
            # 准备四个控件
            # gwg = QWidget()
            # # 使用四个控件设置局部布局
            # gwg.setLayout(glayout)
            # # 将四个控件添加到全局布局中
            # wlayout.addWidget(gwg)
            self.setLayout(glayout)
        except:
            stdinit.std_signal_gobal.stdprintln()



    # cmd_boards  cmd_boards_num cmd_upMethod cmd_xserial cmd_usb cmd_xusb cmd_opt cmd_rtlib
    def load_board(self):
        try:
            if stdinit.pio_boards == None:
                if self.pio_pro.project_boards():
                    num = len(stdinit.pio_boards)
                    boards_list = []
                    for i in range(num):
                        boards_list.append(stdinit.pio_boards[i]['name'])
                    self.board.addItems(boards_list)
                    self.line_label.setText("")

                else:
                    target = stdinit.stdenv + "/.stduino/packages/pioenv/pioboards.json"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
                    if os.path.exists(target):
                        self.line_label.setText("哪里发生了问题！")
                        self.line_label.setStyleSheet("color:DeepSkyBlue")
                        self.project_build.setEnabled(False)
                        stdinit.std_signal_gobal.std_echo_msg(0, "请重启软件后再次执行该操作，若还存在该问题请至插件安装界面卸载并重新安装Platformio！")
                    else:
                        if stdinit.pio_boards == None:
                            #self.line_label.setText("请重新安装Platformio！")
                            self.line_label.setStyleSheet("color:DeepSkyBlue")
                            self.project_build.setEnabled(False)
                            stdinit.std_signal_gobal.std_echo_msg(0,"当前插件安装存在文件缺失，请至插件安装界面卸载并重新安装Platformio！")

                        else:
                            num = len(stdinit.pio_boards)
                            boards_list = []
                            for i in range(num):
                                boards_list.append(stdinit.pio_boards[i]['name'])
                            self.board.addItems(boards_list)
                            #self.line_label.setText("请先安装PlatformIO！")
                            self.line_label.setStyleSheet("color:DeepSkyBlue")
        except:
            stdinit.std_signal_gobal.stdprintln()

        # print(1)


    def load_boards(self):
        try:
            if stdinit.pio_boards == None:
                t1 = threading.Thread(target=self.load_board, name='load_board', args=())
                t1.setDaemon(True)
                t1.start()

        except:
            stdinit.std_signal_gobal.stdprintln()

        #print(1)



    def project_init(self):
        try:
            stdinit.std_signal_gobal.std_process(1, "正在构建")
            target = stdinit.stdenv + "/.stduino/packages/pioenv/pioboards.json"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
            if os.path.exists(target):

                if self.pio_pro.project_init(self.upload_method.currentText(), self.framework.currentText()):
                    stdinit.std_signal_gobal.std_process(0, "构建完成！")
                    stdinit.std_signal_gobal.std_echo_msg(0, "构建完成！")
                else:
                    stdinit.std_signal_gobal.std_process(0, "构建失败！")
                    stdinit.std_signal_gobal.std_echo_msg(0, "构建失败！")
            else:
                stdinit.std_signal_gobal.std_echo_msg(0, "开始安装PlatformIO~")
                if stdinit.Pio_install.pio_install():
                    stdinit.std_signal_gobal.std_echo_msg(0, "PlatformIO安装成功！")
                    if self.pio_pro.project_init(self.upload_method.currentText(), self.framework.currentText()):
                        stdinit.std_signal_gobal.std_process(0, "构建完成！")
                        stdinit.std_signal_gobal.std_echo_msg(0, "构建完成！")
                    else:
                        stdinit.std_signal_gobal.std_process(0, "构建失败！")
                        stdinit.std_signal_gobal.std_echo_msg(0, "构建失败！")
                else:
                    stdinit.std_signal_gobal.std_echo_msg(0, "PlatformIO安装失败，暂不可构建项目！")
                    stdinit.std_signal_gobal.std_process(0, "构建失败！")

        except:
            stdinit.std_signal_gobal.stdprintln()


    def project_build_f(self):
        try:
            if self.project_nameEdit.text() == "":
                QMessageBox.warning(self, "Warning",
                                    "警告：请输入项目名称!\n",
                                    QMessageBox.Yes)
            else:
                try:
                    stdinit.project_name = self.project_nameEdit.text()
                    t1 = threading.Thread(target=self.project_init, name='load_board', args=())
                    t1.setDaemon(True)
                    t1.start()
                    stdinit.find_boards.close()
                    stdinit.std_signal_gobal.std_find_staus_change()
                except:
                    stdinit.std_signal_gobal.stdprintln()
                    stdinit.find_boards.close()
                    stdinit.std_signal_gobal.std_find_staus_change()

        except:
            stdinit.std_signal_gobal.stdprintln()



        # if stdinit.pio_boards == None:
        #     t1 = threading.Thread(target=self.load_board, name='load_board', args=())
        #     t1.setDaemon(True)
        #     t1.start()
        # else:
        #     pass
    def project_build_c(self):
        try:
            stdinit.find_boards.close()
            stdinit.std_signal_gobal.std_find_staus_change()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def to_board(self):


        try:
            text = self.board.currentIndex()
            stdinit.board_id = stdinit.pio_boards[text]['id']
            self.project_build.setEnabled(True)
            num = len(stdinit.pio_boards[text]['frameworks'])
            self.framework.clear()
            for i in range(num):
                self.framework.addItem(stdinit.pio_boards[text]['frameworks'][i])

            if 'debug' in stdinit.pio_boards[text]:
                self.upload_method.clear()
                for key, value in stdinit.pio_boards[text]['debug']['tools'].items():
                    self.upload_method.addItem(key)
            else:
                self.upload_method.clear()
                self.upload_method.addItem("Disable")


            # self.staus_change.emit(cmd_args["cmd_boards_num"], cmd_args["cmd_upMethod"])
            #             # os.chdir(pos)
        except:
            stdinit.std_signal_gobal.stdprintln()




# class Model(object):
#     def logic(self):
#         data = "Got it"
#         print("Model: Crunching data as per business logic")
#         return data
#
#
# class View(object):
#     def updata(self, data):
#         print("View:Updating the view with results: ", data)
#
#
# class Controller(object):
#     def __init__(self):
#         self.model = Model()
#         self.view = View()
#
#     def interface(self):
#         print("Controller: Relayed the Client asks")
#         data = self.model.logic()
#         self.view.updata(data)


# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
import PyQt5.QtPrintSupport
from PyQt5.QtGui import QColor,QFont
from PyQt5.Qsci import QsciScintilla,QsciLexerCPP
from function.conf import res
from function.cores.stdedit import stdinit

global key_load_high
key_load_high=0
global key_load
key_load=""
global auto_key_l
auto_key_l=0
global auto_key_list
auto_key_list=[]
class QsciLexerC(QsciLexerCPP):
    def __init__(self, parent):
        super(QsciLexerC, self).__init__(parent)
        try:

            #
            # Initialize colors per style
            # ----------------------------
            # self.setColor(QColor("#ff" + "000000"), 0)  # black
            # self.setColor(QColor("#ff" + "000000"), 1)  # black <b>
            # self.setColor(QColor("#ff" + "d60404"), 2)  # red
            # self.setColor(QColor("#ff" + "d60404"), python3)  # red <b>
            # self.setColor(QColor("#ff" + "ff7f00"), 4)  # orange
            # self.setColor(QColor("#ff" + "ff7f00"), 5)  # orange <b>
            # self.setColor(QColor("#ff" + "ba9b00"), 6)  # yellow
            # self.setColor(QColor("#ff" + "ba9b00"), 7)  # yellow <b>
            # self.setColor(QColor("#ff" + "20ad20"), 8)  # lightgreen
            # self.setColor(QColor("#ff" + "20ad20"), 9)  # lightgreen <b>
            # self.setColor(QColor("#ff" + "005900"), 10)  # green
            # self.setColor(QColor("#ff" + "005900"), 11)  # green <b>
            # self.setColor(QColor("#ff" + "0202ce"), 12)  # blue
            # self.setColor(QColor("#ff" + "0202ce"), 13)  # blue <b>
            # self.setColor(QColor("#ff" + "9400d3"), 14)  # lightpurple
            # self.setColor(QColor("#ff" + "9400d3"), 15)  # lightpurple <b>
            # self.setColor(QColor("#ff" + "4b0082"), 16)  # purple
            # self.setColor(QColor("#ff" + "4b0082"), 17)  # purple <b>
            # self.setColor(QColor("#ff" + "0668d1"), 18)  # cyan
            # self.setColor(QColor("#ff" + "0668d1"), 19)  # cyan <b>
            # #
            # # # Initialize paper colors per style
            # # ----------------------------------

            #
            # # Initialize fonts per style
            # # ---------------------------
            # self.setFont(QFont("Consolas", 13, ), 0)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), 1)
            # self.setFont(QFont("Consolas", 13, ), 2)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), python3)
            # self.setFont(QFont("Consolas", 13, ), 4)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), 5)
            # self.setFont(QFont("Consolas", 13, ), 6)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), 7)
            # self.setFont(QFont("Consolas", 12, ), 8)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), 9)
            # self.setFont(QFont("Consolas", 13, ), 10)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), 11)
            # self.setFont(QFont("Consolas", 13, ), 12)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), 13)
            # self.setFont(QFont("Consolas", 13, ), 14)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), 15)
            # self.setFont(QFont("Consolas", 13, ), 16)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), 17)
            # self.setFont(QFont("Consolas", 13, ), 18)
            # self.setFont(QFont("Consolas", 13, weight=QFont.Bold), 19)

            self.setDefaultFont(QFont("Consolas", 14))  # 字体大小及样式
            self.setDefaultColor(QColor(res.default_text))  # 普通文字背景
            self.setDefaultPaper(QColor(res.default_main))  # 背景  #363636  95%背景颜色#363636
            self.setPaper(QColor(res.default_main), 0)  # Style 0: #363636  python3% 空格背景颜色
            self.setPaper(QColor(res.default_main), 1)  # Style 0: #363636  python3% 空格背景颜色
            self.setPaper(QColor(res.default_main), 2)  # Style 2: #363636  python3% 空格背景颜色
            self.setPaper(QColor(res.default_main), 3)  # Style 0: #363636  python3% 空格背景颜色
            self.setPaper(QColor(res.default_main), 4)  # Style 0: #363636  python3% 空格背景颜色
            self.setPaper(QColor(res.default_main), 5)  # Style 2: #363636  python3% 空格背景颜色
            self.setPaper(QColor(res.default_main), 6)  # Style 2: #363636  python3% 空格背景颜色

            # Initialize fonts per style
            # ---------------------------
            # self.setFont(QFont('华文楷体', 10))
            c_conmmert = QFont("Consolas", 14, weight=QFont.Bold)
            c_conmmert.setItalic(True)  # 斜体

            self.setFont(QFont("Consolas", 14, weight=QFont.Bold), 0)  # Style 0: Consolas 14pt
            self.setFont(QFont("Consolas", 14, weight=QFont.Bold), 1)  # Style 1: Consolas 14pt
            self.setFont(c_conmmert, 2)  # Style 2: Consolas 14pt
            self.setFont(c_conmmert, 3)  # Style python3: Consolas 14pt
            self.setFont(QFont("Consolas", 14, weight=QFont.Bold), 6)  # Style 1: Consolas 14pt
            self.setColor(QColor(res.default_mainkey1), 5)  # 主关键字
            self.setColor(QColor(res.default_mainkey2), 16)  # 008B45 主关键字2
            self.setColor(QColor(res.default_kh), 10)  # 点颜色 分号及括号颜色 运算符
            self.setColor(QColor(res.default_comment), 2)  # 行注释
            self.setColor(QColor(res.default_comment), 3)  # 块注释
            self.setColor(QColor(res.default_num), 4)  # 数字
            self.setColor(QColor(res.default_zh_hans), 6)  # 汉语
            self.setColor(QColor(res.default_include), 9)  # 预编译
            self.setColor(QColor(res.default_include), 19)  # 预编译
            self.setColor(QColor(res.default_include), 23)  # 预编译
            self.setColor(QColor(res.default_include), 24)  # 预编译


            self.setColor(QColor(res.default_include), 7)  # 单引号




            #
            self.setFoldAtElse(True)
            self.setFoldComments(True)
            self.setFoldCompact(False)
            self.setFoldPreprocessor(True)
            self.setStylePreprocessor(True)


        except:
            stdinit.std_signal_gobal.stdprintln()
            ''''''







    def key_load_h(self):

        try:
            # 关键词1
            key_load1 = ""
            try:
                pass
                # for root, dirs, files in os.walk(res.library):
                #     for name in files:
                #         if name == 'keywords.txt':
                #
                #             try:
                #                 f = open(os.path.join(root, name))  # 返回一个文件对象
                #                 # line = f.readline()  # 调用文件的 readline()方法
                #                 lines = f.readlines()  # 调用文件的 readline()方法
                #                 for line in lines:
                #
                #
                #                     if "#" in line or line == "\n":
                #
                #                         pass
                #                     else:
                #
                #                         key_load1 += line.replace("\n", " ")
                #
                #                 f.close()
                #                 # key2 += "\n"
                #             except:
                #                 print("Unexpected error:", sys.exc_info()[1])  # 错误内容
                                # QMessageBox.warning(self, "BUG Warning",
                                #                     "Waring|Error:00013！\n You can search it in stduino.com",
                                #                     QMessageBox.Yes)
                pass
            except:
                stdinit.std_signal_gobal.stdprintln()

               # QMessageBox.warning(self, "BUG Warning", "Waring|Error:00014！\n You can search it in stduino.com",
               #                      QMessageBox.Yes)
            key_load1 = key_load1 + " setup loop HIGH LOW INPUT available read begin print println OUTPUT INPUT_PULSE INPUT_PULLUP INPUT_ANALOG OUTPUT_PULSE " \
                          "delay delayMicroseconds pinMode digitalWrite digitalRead analogRead analogWrite millis micros random " \
                          "map tone noTone if else while for switch case do break continue return goto min max abs constrain " \
                          "sizeof static const pow sqrt peek peekString"
            #print(key_load1)
            return key_load1
        except:
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:00015！\n You can search it in stduino.com",QMessageBox.Yes)
            stdinit.std_signal_gobal.stdprintln()
            pass
        # try:
        #     # 关键词1
        #     if p_int == 1:
        #         # print(13)
        #         return "int void double float boolean String char unsigned long Serial  Serial1 Serial2 Serial3 Serial4"
        #
        #     # 关键词2
        #     if p_int == 2:
        #         # print(python3)
        #         key2 = ""
        #         try:
        #             for root, dirs, files in os.walk(res.library):
        #                 for name in files:
        #                     if name == 'keywords.txt':
        #
        #
        #                         try:
        #                             f = open(os.path.join(root, name))  # 返回一个文件对象
        #                             # line = f.readline()  # 调用文件的 readline()方法
        #                             lines = f.readlines()  # 调用文件的 readline()方法
        #                             for line in lines:
        #
        #                                if "#" in line or line=="\n":
        #
        #                                    pass
        #                                else:
        #
        #                                    key2 += line.replace("\n", " ")
        #
        #
        #                             f.close()
        #                             # key2 += "\n"
        #                         except:
        #                             #print("Unexpected error:", sys.exc_info()[1])  # 错误内容
        #                             QMessageBox.warning(self, "BUG Warning", "Waring|Error:00013！\n You can search it in stduino.com",QMessageBox.Yes)
        #             pass
        #         except:
        #             QMessageBox.warning(self, "BUG Warning", "Waring|Error:00014！\n You can search it in stduino.com",QMessageBox.Yes)
        #         key2 = key2 + " setup loop HIGH LOW INPUT available read begin print println OUTPUT INPUT_PULSE INPUT_PULLUP INPUT_ANALOG OUTPUT_PULSE " \
        #                       "delay delayMicroseconds pinMode digitalWrite digitalRead analogRead analogWrite millis micros random " \
        #                       "map tone noTone if else while for switch case do break continue return goto min max abs constrain " \
        #                       "sizeof static const pow sqrt peek peekString"
        #         return key2
        # except:
        #     QMessageBox.warning(self, "BUG Warning", "Waring|Error:00015！\n You can search it in stduino.com",QMessageBox.Yes)
        #     # print("Unexpected error:", sys.exc_info()[1])  # 错误内容
        #     pass


    def keywords(self, p_int):
        try:
            global key_load_high
            global key_load
            if key_load_high == 0:

                key_load_high = 1
                key_load=self.key_load_h()

            # key_load = threading.Thread(target=self.key_load_h, name='key_load_hi')
            # # key_load.setDaemon(True)
            # key_load.start()
            if p_int == 1:
                # print(13)
                return "int void double float boolean String char unsigned long Serial  Serial1 Serial2 Serial3 Serial4"
            if p_int == 2:
                # print(python3)
                key2 = key_load
                return key2
        except:
            stdinit.std_signal_gobal.stdprintln()



import requests
import json
import urllib.request
from shutil import rmtree
import os,stat
import tarfile
import glob
import re

def is_connected():
    try:
        requests.get("https://www.baidu.com", timeout=2)
    except:
        return False
    return True
def load_json(target_abs):
    try:
        fo = open(target_abs, mode='r', encoding='UTF-8')
        pio_boards =json.loads(fo.read())  # 字符串转json
        fo.close()  # cmd_args["cmd_rtlib"]
        return pio_boards
    except:
        return []

def path_endswith_ext(path, extensions):
    if not isinstance(extensions, (list, tuple)):
        extensions = [extensions]
    for ext in extensions:
        if path.endswith("." + ext):
            return True
    return False
def match_src_files(src_dir, src_filter=None, src_exts=None, followlinks=True):
    def _append_build_item(items, item, src_dir):
        if not src_exts or path_endswith_ext(item, src_exts):
            items.add(os.path.relpath(item, src_dir))

    src_filter = src_filter or ""
    if isinstance(src_filter, (list, tuple)):
        src_filter = " ".join(src_filter)

    matches = set()
    # correct fs directory separator
    src_filter = src_filter.replace("/", os.sep).replace("\\", os.sep)
    for (action, pattern) in re.findall(r"(\+|\-)<([^>]+)>", src_filter):
        items = set()
        for item in glob.glob(
            os.path.join(glob.escape(src_dir), pattern), recursive=True
        ):
            if os.path.isdir(item):
                for root, _, files in os.walk(item, followlinks=followlinks):
                    for f in files:
                        _append_build_item(items, os.path.join(root, f), src_dir)
            else:
                _append_build_item(items, item, src_dir)
        if action == "+":
            matches |= items
        else:
            matches -= items
    return sorted(list(matches))
#一次性打包整个根目录。空子目录会被打包。
#如果只打包不压缩，将"w:gz"参数改为"w:"或"w"即可。
def make_targz(output_filename, source_dir):
    with tarfile.open(output_filename, "w:gz") as tar:
        tar.add(source_dir,arcname="")#,


def untar_file(filename,tarpath):
    t = tarfile.open(filename)
    t.extractall(path=tarpath)



def fast_download(url,target_path):
    try:
        urllib.request.urlretrieve(url, target_path)
    except:
        return False
    return True

def remove_readonly(func, path, _):
    try:
        # "Clear the readonly bit and reattempt the removal"
        os.chmod(path, stat.S_IWRITE)
        func(path)
    except:
        print("something wrong of file delete")


def delete_file(path):
    try:
        if os.path.isdir(path):
            try:
                rmtree(path, onerror=remove_readonly)
            except:
                print("something wrong of file delete")
        else:
            try:
                os.remove(path)
            except:
                try:
                    # shutil.rmtree(path, onerror=self.remove_readonly)
                    os.chmod(path, stat.S_IWRITE)
                    os.remove(path)
                except:
                    print("something wrong of file delete")
    except:
        print("something wrong of file delete")

if __name__ == '__main__':
    ss="C:/Users/debug/.stduino/platforms/w80x"
    name="C:/Users/debug/.stduino/platforms/w80x.tar.gz"
    make_targz(name,ss)


# Copyright (c) 2014-present PlatformIO <contact@platformio.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import fnmatch
import os
import sys

from SCons import Builder, Util  # pylint: disable=import-error
from SCons.Node import FS  # pylint: disable=import-error
from SCons.Script import COMMAND_LINE_TARGETS  # pylint: disable=import-error
from SCons.Script import AlwaysBuild  # pylint: disable=import-error
from SCons.Script import DefaultEnvironment  # pylint: disable=import-error
from SCons.Script import Export  # pylint: disable=import-error
from SCons.Script import SConscript  # pylint: disable=import-error

from platformio import __version__, fs
from platformio.compat import IS_MACOS, string_types
from platformio.package.version import pepver_to_semver

SRC_HEADER_EXT = ["h", "hpp"]
SRC_ASM_EXT = ["S", "spp", "SPP", "sx", "s", "asm", "ASM"]
SRC_C_EXT = ["c"]
SRC_CXX_EXT = ["cc", "cpp", "cxx", "c++"]
SRC_BUILD_EXT = SRC_C_EXT + SRC_CXX_EXT + SRC_ASM_EXT
SRC_FILTER_DEFAULT = ["+<*>", "-<.git%s>" % os.sep, "-<.svn%s>" % os.sep]


def scons_patched_match_splitext(path, suffixes=None):
    """Patch SCons Builder, append $OBJSUFFIX to the end of each target"""
    tokens = Util.splitext(path)
    if suffixes and tokens[1] and tokens[1] in suffixes:
        return (path, tokens[1])
    return tokens


def GetBuildType(env):
    return (
        "debug"
        if (
            set(["__debug", "sizedata"]) & set(COMMAND_LINE_TARGETS)
            or env.GetProjectOption("build_type") == "debug"
        )
        else "release"
    )


def BuildProgram(env):
    env.ProcessProgramDeps()
    env.ProcessProjectDeps()

    # append into the beginning a main LD script
    if env.get("LDSCRIPT_PATH") and not any("-Wl,-T" in f for f in env["LINKFLAGS"]):
        env.Prepend(LINKFLAGS=["-T", env.subst("$LDSCRIPT_PATH")])

    # enable "cyclic reference" for linker
    if (
        env.get("LIBS")
        and env.GetCompilerType() == "gcc"
        and (env.PioPlatform().is_embedded() or not IS_MACOS)
    ):
        env.Prepend(_LIBFLAGS="-Wl,--start-group ")
        env.Append(_LIBFLAGS=" -Wl,--end-group")

    program = env.Program(
        os.path.join("$BUILD_DIR", env.subst("$PROGNAME$PROGSUFFIX")),
        env["PIOBUILDFILES"],
    )
    env.Replace(PIOMAINPROG=program)

    AlwaysBuild(
        env.Alias(
            "checkprogsize",
            program,
            env.VerboseAction(env.CheckUploadSize, "Checking size $PIOMAINPROG"),
        )
    )

    print("Building in %s mode" % env.GetBuildType())

    return program


def ProcessProgramDeps(env):
    def _append_pio_macros():
        core_version = pepver_to_semver(__version__)
        env.AppendUnique(
            CPPDEFINES=[
                (
                    "PLATFORMIO",
                    int(
                        "{0:02d}{1:02d}{2:02d}".format(
                            core_version.major, core_version.minor, core_version.patch
                        )
                    ),
                )
            ]
        )

    _append_pio_macros()

    env.PrintConfiguration()

    # fix ASM handling under non case-sensitive OS
    if not Util.case_sensitive_suffixes(".s", ".S"):
        env.Replace(AS="$CC", ASCOM="$ASPPCOM")

    # process extra flags from board
    if "BOARD" in env and "build.extra_flags" in env.BoardConfig():
        env.ProcessFlags(env.BoardConfig().get("build.extra_flags"))

    # apply user flags
    env.ProcessFlags(env.get("BUILD_FLAGS"))

    # process framework scripts
    env.BuildFrameworks(env.get("PIOFRAMEWORK"))

    if env.GetBuildType() == "debug":
        env.ConfigureDebugFlags()

    # remove specified flags
    env.ProcessUnFlags(env.get("BUILD_UNFLAGS"))

    if "__test" in COMMAND_LINE_TARGETS:
        env.ConfigureTestTarget()


def ProcessProjectDeps(env):
    project_lib_builder = env.ConfigureProjectLibBuilder()

    # prepend project libs to the beginning of list
    env.Prepend(LIBS=project_lib_builder.build())
    # prepend extra linker related options from libs
    env.PrependUnique(
        **{
            key: project_lib_builder.env.get(key)
            for key in ("LIBS", "LIBPATH", "LINKFLAGS")
            if project_lib_builder.env.get(key)
        }
    )

    projenv = env.Clone()

    # CPPPATH from dependencies
    projenv.PrependUnique(CPPPATH=project_lib_builder.env.get("CPPPATH"))
    # extra build flags from `platformio.ini`
    projenv.ProcessFlags(env.get("SRC_BUILD_FLAGS"))

    is_test = "__test" in COMMAND_LINE_TARGETS
    if is_test:
        projenv.BuildSources(
            "$BUILD_TEST_DIR", "$PROJECT_TEST_DIR", "$PIOTEST_SRC_FILTER"
        )
    if not is_test or env.GetProjectOption("test_build_project_src"):
        projenv.BuildSources(
            "$BUILD_SRC_DIR", "$PROJECT_SRC_DIR", env.get("SRC_FILTER")
        )

    if not env.get("PIOBUILDFILES") and not COMMAND_LINE_TARGETS:
        sys.stderr.write(
            "Error: Nothing to build. Please put your source code files "
            "to '%s' folder\n" % env.subst("$PROJECT_SRC_DIR")
        )
        env.Exit(1)

    Export("projenv")


def ParseFlagsExtended(env, flags):  # pylint: disable=too-many-branches
    if not isinstance(flags, list):
        flags = [flags]
    result = {}
    for raw in flags:
        for key, value in env.ParseFlags(str(raw)).items():
            if key not in result:
                result[key] = []
            result[key].extend(value)

    cppdefines = []
    for item in result["CPPDEFINES"]:
        if not Util.is_Sequence(item):
            cppdefines.append(item)
            continue
        name, value = item[:2]
        if '"' in value:
            value = value.replace('"', '\\"')
        elif value.isdigit():
            value = int(value)
        elif value.replace(".", "", 1).isdigit():
            value = float(value)
        cppdefines.append((name, value))
    result["CPPDEFINES"] = cppdefines

    # fix relative CPPPATH & LIBPATH
    for k in ("CPPPATH", "LIBPATH"):
        for i, p in enumerate(result.get(k, [])):
            if os.path.isdir(p):
                result[k][i] = os.path.realpath(p)

    # fix relative path for "-include"
    for i, f in enumerate(result.get("CCFLAGS", [])):
        if isinstance(f, tuple) and f[0] == "-include":
            result["CCFLAGS"][i] = (f[0], env.File(os.path.realpath(f[1].get_path())))

    return result


def ProcessFlags(env, flags):  # pylint: disable=too-many-branches
    if not flags:
        return
    env.Append(**env.ParseFlagsExtended(flags))

    # Cancel any previous definition of name, either built in or
    # provided with a -U option // Issue #191
    undefines = [
        u
        for u in env.get("CCFLAGS", [])
        if isinstance(u, string_types) and u.startswith("-U")
    ]
    if undefines:
        for undef in undefines:
            env["CCFLAGS"].remove(undef)
            if undef[2:] in env["CPPDEFINES"]:
                env["CPPDEFINES"].remove(undef[2:])
        env.Append(_CPPDEFFLAGS=" %s" % " ".join(undefines))


def ProcessUnFlags(env, flags):
    if not flags:
        return
    parsed = env.ParseFlagsExtended(flags)

    # get all flags and copy them to each "*FLAGS" variable
    all_flags = []
    for key, unflags in parsed.items():
        if key.endswith("FLAGS"):
            all_flags.extend(unflags)
    for key, unflags in parsed.items():
        if key.endswith("FLAGS"):
            parsed[key].extend(all_flags)

    for key, unflags in parsed.items():
        for unflag in unflags:
            for current in env.get(key, []):
                conditions = [
                    unflag == current,
                    isinstance(current, (tuple, list)) and unflag[0] == current[0],
                ]
                if any(conditions):
                    env[key].remove(current)


def MatchSourceFiles(env, src_dir, src_filter=None):
    src_filter = env.subst(src_filter) if src_filter else None
    src_filter = src_filter or SRC_FILTER_DEFAULT
    return fs.match_src_files(
        env.subst(src_dir), src_filter, SRC_BUILD_EXT + SRC_HEADER_EXT
    )


def CollectBuildFiles(
    env, variant_dir, src_dir, src_filter=None, duplicate=False
):  # pylint: disable=too-many-locals
    sources = []
    variants = []

    src_dir = env.subst(src_dir)
    if src_dir.endswith(os.sep):
        src_dir = src_dir[:-1]

    for item in env.MatchSourceFiles(src_dir, src_filter):
        _reldir = os.path.dirname(item)
        _src_dir = os.path.join(src_dir, _reldir) if _reldir else src_dir
        _var_dir = os.path.join(variant_dir, _reldir) if _reldir else variant_dir

        if _var_dir not in variants:
            variants.append(_var_dir)
            env.VariantDir(_var_dir, _src_dir, duplicate)

        if fs.path_endswith_ext(item, SRC_BUILD_EXT):
            sources.append(env.File(os.path.join(_var_dir, os.path.basename(item))))

    middlewares = env.get("__PIO_BUILD_MIDDLEWARES")
    if not middlewares:
        return sources

    new_sources = []
    for node in sources:
        new_node = node
        for callback, pattern in middlewares:
            if pattern and not fnmatch.fnmatch(node.srcnode().get_path(), pattern):
                continue
            new_node = callback(new_node)
        if new_node:
            new_sources.append(new_node)

    return new_sources


def AddBuildMiddleware(env, callback, pattern=None):
    env.Append(__PIO_BUILD_MIDDLEWARES=[(callback, pattern)])


def BuildFrameworks(env, frameworks):
    if not frameworks:
        return

    if "BOARD" not in env:
        sys.stderr.write(
            "Please specify `board` in `platformio.ini` to use "
            "with '%s' framework\n" % ", ".join(frameworks)
        )
        env.Exit(1)

    board_frameworks = env.BoardConfig().get("frameworks", [])
    if frameworks == ["platformio"]:
        if board_frameworks:
            frameworks.insert(0, board_frameworks[0])
        else:
            sys.stderr.write("Error: Please specify `board` in `platformio.ini`\n")
            env.Exit(1)

    for f in frameworks:
        if f == "arduino":
            # Arduino IDE appends .o the end of filename
            Builder.match_splitext = scons_patched_match_splitext
            if "nobuild" not in COMMAND_LINE_TARGETS:
                env.ConvertInoToCpp()

        if f in board_frameworks:
            SConscript(env.GetFrameworkScript(f), exports="env")
        else:
            sys.stderr.write("Error: This board doesn't support %s framework!\n" % f)
            env.Exit(1)


def BuildLibrary(env, variant_dir, src_dir, src_filter=None, nodes=None):
    env.ProcessUnFlags(env.get("BUILD_UNFLAGS"))
    nodes = nodes or env.CollectBuildFiles(variant_dir, src_dir, src_filter)
    return env.StaticLibrary(env.subst(variant_dir), nodes)


def BuildSources(env, variant_dir, src_dir, src_filter=None):
    nodes = env.CollectBuildFiles(variant_dir, src_dir, src_filter)
    DefaultEnvironment().Append(
        PIOBUILDFILES=[
            env.Object(node) if isinstance(node, FS.File) else node for node in nodes
        ]
    )


def exists(_):
    return True


def generate(env):
    env.AddMethod(GetBuildType)
    env.AddMethod(BuildProgram)
    env.AddMethod(ProcessProgramDeps)
    env.AddMethod(ProcessProjectDeps)
    env.AddMethod(ParseFlagsExtended)
    env.AddMethod(ProcessFlags)
    env.AddMethod(ProcessUnFlags)
    env.AddMethod(MatchSourceFiles)
    env.AddMethod(CollectBuildFiles)
    env.AddMethod(AddBuildMiddleware)
    env.AddMethod(BuildFrameworks)
    env.AddMethod(BuildLibrary)
    env.AddMethod(BuildSources)
    return env
